void PlotBPZH(const Char_t *fname = "results/jsf.pfoanl.zh350.v0.std.10k.root")
//void PlotBPZH(const Char_t *fname = "results/jsf.pfoanl.zh350.v0.tkemhdexact.10k.root")
//void PlotBPZH(const Char_t *fname = "results/jsf.pfoanl.zh350.v0.emhdexact.10k.root")
//void PlotBPZH(const Char_t *fname = "results/jsf.pfoanl.zh350.v0.emexact.10k.root")
//void PlotBPZH(const Char_t *fname = "results/jsf.pfoanl.zh350.v0.hdexact.10k.root")
{
  // ------------------------------------------------------------
  //  Open shared libraries
  // ------------------------------------------------------------
  gSystem->Load("libJSF.so");
  gSystem->Load("libS4Utils.so");
  gSystem->Load("libAnlib.so");

  const Double_t kMh = 120.;

  // ------------------------------------------------------------
  //  Open event file and apply cuts
  // ------------------------------------------------------------
  TFile *filep = TFile::Open(fname);
  TNtupleD *tupep = (TNtupleD *)gROOT->FindObject("hevt");

  //tupep->Draw(">>elist","npj1>=5&&npj2>=5&&abs(cosj1)<0.8&&abs(cosj2)<0.8&&mass>70&&mass<110","goff");
  tupep->Draw(">>elist","npj1>=5&&npj2>=5&&abs(cosj1)<0.8&&abs(cosj2)<0.8","goff");
  //tupep->Draw(">>elist","npj1>=5&&npj2>=5&&abs(cosj1)<0.95&&abs(cosj2)<0.95","goff");
  TEventList *elistp = (TEventList *)gROOT->FindObject("elist");

  const Double_t kNsig   = 2.;	// fit range for gaussian fit in units of sigma
  const Double_t kNsigLo = 1.;	// lower  bound of fit range for gaussian fit in units of sigma
  const Double_t kNsigHi = 2.;	// higher bound of fit range for gaussian fit in units of sigma

  // ------------------------------------------------------------
  //  Define histograms
  // ------------------------------------------------------------
#if 1
  Double_t dEmin = -15.;
  Double_t dEmax = +15.;
  Double_t dEvmn = -20.;
  Double_t dEvmx = +20.;
  Double_t Emin  =   0.;
  //Double_t Emax  = 100.;
  Double_t Emax  = 180.;
  //Double_t Emax  = 200.;
#else
  Double_t dEmin = -30.;
  Double_t dEmax = +30.;
  Double_t dEvmn = -40.;
  Double_t dEvmx = +40.;
  Double_t Emin  =   0.;
  //Double_t Emax  = 500.;
  //Double_t Emax  = 200.;
  //Double_t Emax  = 350.;
  Double_t Emax  = 180.;
#endif
  TH1D *hDetkbp  = new TH1D("hDetkbp" ,"",120,dEmin,dEmax);
  TH1D *hDeembp  = new TH1D("hDeembp" ,"",120,dEmin,dEmax);
  TH1D *hDehdbp  = new TH1D("hDehdbp" ,"",120,dEmin,dEmax);
  TH1D *hDetot   = new TH1D("hDetot"  ,"",100,dEvmn,dEvmx);
  TH1D *hDecm    = new TH1D("hDecm"   ,"",100,dEvmn,dEvmx);
  TH1D *hDmass   = new TH1D("hDmass"  ,"",100,dEvmn,dEvmx);
  TH1D *hDmjjisr = new TH1D("hDmjjisr","",100,dEvmn,dEvmx);
  TH1D *hDmjj    = new TH1D("hDmjj"   ,"",100,dEvmn,dEvmx);
  TH1D *hDmjjnu  = new TH1D("hDmjjnu" ,"",100,dEvmn,dEvmx);
  TH1D *hDmjjnah = new TH1D("hDmjjnah","",100,dEvmn,dEvmx);
  TH1D *hEtkbp   = new TH1D("hEtkbp"  ,"", 90, Emin, Emax);
  TH1D *hEembp   = new TH1D("hEembp"  ,"", 90, Emin, Emax);
  TH1D *hEhdbp   = new TH1D("hEhdbp"  ,"", 90, Emin, Emax);
  TH1D *hEahbp   = new TH1D("hEahbp"  ,"", 90, Emin, Emax);
  TH1D *hEisr    = new TH1D("hEisr"   ,"",300, Emin, Emax/6);
  TH1D *hEahhnu  = new TH1D("hEahhnu" ,"",300, Emin, Emax/3);

  // ------------------------------------------------------------
  //  Loop over selected events and fill up histograms
  // ------------------------------------------------------------
  Int_t nevt = elistp->GetN();
  cerr << "Nevt = " << nevt << endl;

  for (Int_t i=0; i<nevt; i++) {
    Int_t ievt = elistp->GetEntry(i);	// get event number

    Double_t npj1, npj2, cosj1, cosj2, mass;
    Double_t etk, eem, ehd, etkbp, eembp, ehdbp, eah, eahznu, eahisr;
    Double_t etot, pxtot, pytot, pztot, eisr, pxisr, pyisr, pzisr,
	     eahhnu, pxahhnu, pyahhnu, pzahhnu,
	     pxah, pyah, pzah, pxahznu, pyahznu, pzahznu, 
	     pxahisr, pyahisr, pzahisr, mjj;
    Double_t pxem, pyem, pzem, pxhd, pyhd, pzhd;
    Double_t edbltk, pxdbltk, pydbltk, pzdbltk,
             edblem, pxdblem, pydblem, pzdblem,
             edblhd, pxdblhd, pydblhd, pzdblhd;
    Double_t edbltkab, pxdbltkab, pydbltkab, pzdbltkab;
    Double_t edbltkmpc, pxdbltkmpc, pydbltkmpc, pzdbltkmpc;
    Double_t etkmpc, pxtkmpc, pytkmpc, pztkmpc;

    tupep->SetBranchAddress("npj1",&npj1);
    tupep->SetBranchAddress("npj2",&npj2);
    tupep->SetBranchAddress("cosj1",&cosj1);
    tupep->SetBranchAddress("cosj2",&cosj2);
    tupep->SetBranchAddress("mass",&mass);
    tupep->SetBranchAddress("etk",&etk);
    tupep->SetBranchAddress("eem",&eem);
    tupep->SetBranchAddress("ehd",&ehd);
    tupep->SetBranchAddress("etkbp",&etkbp);
    tupep->SetBranchAddress("eembp",&eembp);
    tupep->SetBranchAddress("ehdbp",&ehdbp);
    tupep->SetBranchAddress("eah",&eah);
    tupep->SetBranchAddress("eahznu",&eahznu);
    tupep->SetBranchAddress("eahisr",&eahisr);
    tupep->SetBranchAddress("eisr",&eisr);
    tupep->SetBranchAddress("pxisr",&pxisr);
    tupep->SetBranchAddress("pyisr",&pyisr);
    tupep->SetBranchAddress("pzisr",&pzisr);
    tupep->SetBranchAddress("px",&pxtot);
    tupep->SetBranchAddress("py",&pytot);
    tupep->SetBranchAddress("pz",&pztot);
    tupep->SetBranchAddress("eahhnu",&eahhnu);
    tupep->SetBranchAddress("pxahhnu",&pxahhnu);
    tupep->SetBranchAddress("pyahhnu",&pyahhnu);
    tupep->SetBranchAddress("pzahhnu",&pzahhnu);
    tupep->SetBranchAddress("pxem",&pxem);
    tupep->SetBranchAddress("pyem",&pyem);
    tupep->SetBranchAddress("pzem",&pzem);
    tupep->SetBranchAddress("pxhd",&pxhd);
    tupep->SetBranchAddress("pyhd",&pyhd);
    tupep->SetBranchAddress("pzhd",&pzhd);
    tupep->SetBranchAddress("pxah",&pxah);
    tupep->SetBranchAddress("pyah",&pyah);
    tupep->SetBranchAddress("pzah",&pzah);
    tupep->SetBranchAddress("pxahznu",&pxahznu);
    tupep->SetBranchAddress("pyahznu",&pyahznu);
    tupep->SetBranchAddress("pzahznu",&pzahznu);
    tupep->SetBranchAddress("pxahisr",&pxahisr);
    tupep->SetBranchAddress("pyahisr",&pyahisr);
    tupep->SetBranchAddress("pzahisr",&pzahisr);
    tupep->SetBranchAddress("edbltk",&edbltk);
    tupep->SetBranchAddress("pxdbltk",&pxdbltk);
    tupep->SetBranchAddress("pydbltk",&pydbltk);
    tupep->SetBranchAddress("pzdbltk",&pzdbltk);
    tupep->SetBranchAddress("edblem",&edblem);
    tupep->SetBranchAddress("pxdblem",&pxdblem);
    tupep->SetBranchAddress("pydblem",&pydblem);
    tupep->SetBranchAddress("pzdblem",&pzdblem);
    tupep->SetBranchAddress("edblhd",&edblhd);
    tupep->SetBranchAddress("pxdblhd",&pxdblhd);
    tupep->SetBranchAddress("pydblhd",&pydblhd);
    tupep->SetBranchAddress("pzdblhd",&pzdblhd);
    tupep->SetBranchAddress("edbltkab",&edbltkab);
    tupep->SetBranchAddress("pxdbltkab",&pxdbltkab);
    tupep->SetBranchAddress("pydbltkab",&pydbltkab);
    tupep->SetBranchAddress("pzdbltkab",&pzdbltkab);
    tupep->SetBranchAddress("edbltkmpc",&edbltkmpc);
    tupep->SetBranchAddress("pxdbltkmpc",&pxdbltkmpc);
    tupep->SetBranchAddress("pydbltkmpc",&pydbltkmpc);
    tupep->SetBranchAddress("pzdbltkmpc",&pzdbltkmpc);
    tupep->SetBranchAddress("etkmpc",&etkmpc);
    tupep->SetBranchAddress("pxtkmpc",&pxtkmpc);
    tupep->SetBranchAddress("pytkmpc",&pytkmpc);
    tupep->SetBranchAddress("pztkmpc",&pztkmpc);

    tupep->GetEntry(ievt);	// get this event

    etot = etk + eem + ehd;
    ANL4DVector qtot(etot, pxtot, pytot, pztot); // after eliminating double counting.
    ANL4DVector qhd (ehd , pxhd , pyhd , pzhd ); // neutral hadrons

    if (TString(fname).Contains("hdexact")) {
      const Double_t kFcorr = 1.0;
    } else {
      const Double_t kFcorr = 1.065;	// ad hoc calibration to make <dEhd> = 0.
      //const Double_t kFcorr = 1.0308;	// calibration by A.Miyamoto
    }
    ANL4DVector qhdc = kFcorr * qhd;             // neutral hadrons after recalibration
    Double_t    ehdc = qhdc.E();

    ANL4DVector qdbltk(edbltk, pxdbltk, pydbltk, pzdbltk);
    ANL4DVector qdblem(edblem, pxdblem, pydblem, pzdblem);
    ANL4DVector qdblhd(edblhd, pxdblhd, pydblhd, pzdblhd);
    ANL4DVector qdblhdc = kFcorr * qdblhd;       // double counted neutral hadrons after recalibration
#if 0
    ANL4DVector qtotc = qtot - qhd + qhdc;
    ANL4DVector qvisc   = qtotc + qdbltk + qdblem + qdblhdc;
    Double_t    etkc = etk;
#else
#if 1
    // remove albedo tracks
    ANL4DVector qtotc = qtot - qhd + qhdc;
    //ANL4DVector qvisc   = qtotc + qdbltk + qdblem;
    //ANL4DVector qvisc   = qtotc + qdbltk;
    //ANL4DVector qvisc   = qtotc + qdblem;
    //ANL4DVector qvisc   = qtotc + qdblhd;
    //ANL4DVector qvisc   = qtotc + qdblem + qdblhd;
    ANL4DVector qdbltkab(edbltkab, pxdbltkab, pydbltkab, pzdbltkab);
    ANL4DVector qvisc   = qtotc + qdbltk - qdbltkab + qdblem + qdblhdc;
    Double_t    etkc = etk;
#else
    // remove albedo tracks and correct for 2ndary proton mass
    ANL4DVector qem      (eem , pxem , pyem , pzem ); // em PFOs
    ANL4DVector qtkmpc   (etkmpc, pxtkmpc, pytkmpc, pztkmpc); // tk PFOs with Mp corr.
    ANL4DVector qdbltkmpc(edbltkmpc, pxdbltkmpc, pydbltkmpc, pzdbltkmpc); // double counted tk with Mp corr.

    ANL4DVector qtotc   = qtkmpc + qem + qhdc;
    ANL4DVector qdbltkab(edbltkab, pxdbltkab, pydbltkab, pzdbltkab);
    ANL4DVector qvisc   = qtotc + qdbltkmpc - qdbltkab + qdblem + qdblhdc;
    Double_t    etkc = etkmpc;
#endif
#endif

    ANL4DVector qisr(eisr, pxisr, pyisr, pzisr); // visible isr photons
    ANL4DVector qjj = qtotc - qisr;               // final-state with isr photons eliminated
    ANL4DVector qhnu(eahhnu, pxahhnu, pyahhnu, pzahhnu); // neutrinos from H
    ANL4DVector qjjnu = qjj + qhnu;                      // H corrected for missing neutrinnos
    ANL4DVector qznu(eahznu, pxahznu, pyahznu, pzahznu); // neutrinos from Z -> nu nu
    ANL4DVector qah (eah, pxah, pyah, pzah);             // all undetected 
    ANL4DVector qahi(eahisr, pxahisr, pyahisr, pzahisr); // undetected isr photons
    ANL4DVector qahh = qah - qznu -qhnu - qahi;          // othewise visible particles from H in A.H.
    ANL4DVector qjjnuah = qjjnu + qahh;                  // H corrected for nu and ah

    hDetkbp ->Fill(etkc-etkbp,1.);
    hDeembp ->Fill(eem-eembp,1.);
    hDehdbp ->Fill(ehdc-ehdbp,1.);
    hDetot  ->Fill(etkc+eem+ehdc-etkbp-eembp-ehdbp,1.);
    hDecm   ->Fill(etkc+eem+ehdc-etkbp-eembp-ehdbp-eah+eahznu+eahisr,1.);
    hDmass  ->Fill(qvisc.Mag()-kMh,1.);
    hDmjjisr->Fill(qtotc.Mag()-kMh,1.);
    hDmjj   ->Fill(qjj.Mag() -kMh,1.);
    hDmjjnu ->Fill(qjjnu.Mag() -kMh,1.);
    hDmjjnah->Fill(qjjnuah.Mag() -kMh,1.);
    hEtkbp  ->Fill(etkbp,1.);
    hEembp  ->Fill(eembp,1.);
    hEhdbp  ->Fill(ehdbp,1.);
    hEahbp  ->Fill(eah-eahznu-eahisr,1.);
    hEahhnu ->Fill(eahhnu,1.);
    hEisr   ->Fill(eisr,1.);
  }
  // ------------------------------------------------------------
  //  Draw histograms
  // ------------------------------------------------------------
  gStyle->SetOptFit(1);
  gStyle->SetOptStat(111111);

  //TCanvas *c1p = new TCanvas("c1","",20,20,1220,920);
  TCanvas *c1p = new TCanvas("c1","",20,20,1420,1070);
  TCanvas &c1  = *c1p;
  c1.Divide(4,3);
  Int_t id = 0;

  TPad *cpadp;
  id++; cpadp = (TPad *)c1.cd(id);
  cpadp->SetFillColor(30);
  cpadp->SetFrameFillColor(TColor::GetColor("#fbf7ca"));
  hDetkbp->Draw();
  ANLGaussFitter gfit(hDetkbp,kNsig);
  gfit.GetFunction()->Draw("same");
  hDetkbp->SetFillColor(42);
  hDetkbp->SetFillStyle(1000);
  hDetkbp->GetXaxis()->SetTitle("#Delta E_{tk} [GeV]");

  id++; cpadp = (TPad *)c1.cd(id);
  cpadp->SetFillColor(30);
  cpadp->SetFrameFillColor(TColor::GetColor("#fbf7ca"));
  hDeembp->Draw();
  ANLGaussFitter gfit(hDeembp,kNsig);
  gfit.GetFunction()->Draw("same");
  hDeembp->SetFillColor(50);
  hDeembp->SetFillStyle(3001);
  hDeembp->GetXaxis()->SetTitle("#Delta E_{em} [GeV]");

  id++; cpadp = (TPad *)c1.cd(id);
  cpadp->SetFillColor(30);
  cpadp->SetFrameFillColor(TColor::GetColor("#fbf7ca"));
  hDehdbp->Draw();
  ANLGaussFitter gfit(hDehdbp,kNsig);
  gfit.GetFunction()->Draw("same");
  hDehdbp->SetFillColor(9);
  hDehdbp->SetFillStyle(3002);
  hDehdbp->GetXaxis()->SetTitle("#Delta E_{hd} [GeV]");

  id++; cpadp = (TPad *)c1.cd(id);
  cpadp->SetFillColor(30);
  cpadp->SetFrameFillColor(TColor::GetColor("#fbf7ca"));
  hEtkbp->GetXaxis()->SetTitle("E_{bp} [GeV]");

  hEtkbp->SetFillColor(42);
  hEtkbp->SetFillStyle(3001);
  hEtkbp->Draw();
  hEtkbp->SetMaximum(1000);

  hEembp->SetFillColor(50);
  hEembp->SetFillStyle(3001);
  hEembp->Draw("same");

  hEhdbp->SetFillColor(9);
  hEhdbp->SetFillStyle(3002);
  hEhdbp->Draw("same");

  hEahbp->SetFillColor(38);
  hEahbp->SetFillStyle(3001);
  hEahbp->Draw("same");

  id++; cpadp = (TPad *)c1.cd(id);
  cpadp->SetFillColor(30);
  cpadp->SetFrameFillColor(TColor::GetColor("#fbf7ca"));
  cpadp->SetLogy();
  hEahhnu->GetXaxis()->SetTitle("E_{#nu from H} [GeV]");
  hEahhnu->SetFillColor(38);
  hEahhnu->SetFillStyle(3001);
  hEahhnu->Draw("same");

  id++; cpadp = (TPad *)c1.cd(id);
  cpadp->SetFillColor(30);
  cpadp->SetFrameFillColor(TColor::GetColor("#fbf7ca"));
  cpadp->SetLogy();
  hEisr->GetXaxis()->SetTitle("E_{#gamma_{isr}} [GeV]");
  hEisr->SetFillColor(38);
  hEisr->SetFillStyle(3001);
  hEisr->Draw("same");

  id++; cpadp = (TPad *)c1.cd(id);
  cpadp->SetFillColor(30);
  cpadp->SetFrameFillColor(TColor::GetColor("#fbf7ca"));
  hDetot->Draw();
  ANLGaussFitter gfit(hDetot,kNsig);
  gfit.GetFunction()->Draw("same");
  hDetot->SetFillColor(50);
  hDetot->SetFillStyle(1000);
  hDetot->GetXaxis()->SetTitle("#Delta E_{vis} [GeV]");

#if 0
  id++; cpadp = (TPad *)c1.cd(id);
  cpadp->SetFillColor(30);
  cpadp->SetFrameFillColor(TColor::GetColor("#fbf7ca"));
  hDecm->Draw();
  ANLGaussFitter gfit(hDecm,kNsig);
  gfit.GetFunction()->Draw("same");
  hDecm->SetFillColor(50);
  hDecm->SetFillStyle(1000);
  hDecm->GetXaxis()->SetTitle("E_{vis}+E_{Z}-E_{cm} [GeV]");
#endif

  id++; cpadp = (TPad *)c1.cd(id);
  cpadp->SetFillColor(30);
  cpadp->SetFrameFillColor(TColor::GetColor("#fbf7ca"));
  hDmass->Draw();
  ANLGaussFitter gfit(hDmass,kNsigLo,kNsigHi,-1.e20,1.e20);
  gfit.GetFunction()->Draw("same");
  hDmass->SetFillColor(50);
  hDmass->SetFillStyle(1000);
  hDmass->GetXaxis()->SetTitle("#Delta Mass [GeV]");

  id++; cpadp = (TPad *)c1.cd(id);
  cpadp->SetFillColor(30);
  cpadp->SetFrameFillColor(TColor::GetColor("#fbf7ca"));
  hDmjjisr->Draw();
  ANLGaussFitter gfit(hDmjjisr,kNsigLo,kNsigHi,-1.e20,1.e20);
  gfit.GetFunction()->Draw("same");
  hDmjjisr->SetFillColor(50);
  hDmjjisr->SetFillStyle(1000);
  hDmjjisr->GetXaxis()->SetTitle("#Delta Mass [GeV]");

  id++; cpadp = (TPad *)c1.cd(id);
  cpadp->SetFillColor(30);
  cpadp->SetFrameFillColor(TColor::GetColor("#fbf7ca"));
  hDmjj->Draw();
  ANLGaussFitter gfit(hDmjj,kNsigLo,kNsigHi,-1.e20,1.e20);
  gfit.GetFunction()->Draw("same");
  hDmjj->SetFillColor(50);
  hDmjj->SetFillStyle(1000);
  hDmjj->GetXaxis()->SetTitle("#Delta M_{JJ} [GeV]");

  id++; cpadp = (TPad *)c1.cd(id);
  cpadp->SetFillColor(30);
  cpadp->SetFrameFillColor(TColor::GetColor("#fbf7ca"));
  hDmjjnu->Draw();
  ANLGaussFitter gfit(hDmjjnu,kNsig);
  gfit.GetFunction()->Draw("same");
  hDmjjnu->SetFillColor(50);
  hDmjjnu->SetFillStyle(1000);
  hDmjjnu->GetXaxis()->SetTitle("#Delta M_{JJ} [GeV]");

  id++; cpadp = (TPad *)c1.cd(id);
  cpadp->SetFillColor(30);
  cpadp->SetFrameFillColor(TColor::GetColor("#fbf7ca"));
  hDmjjnah->Draw();
  ANLGaussFitter gfit(hDmjjnah,kNsig);
  gfit.GetFunction()->Draw("same");
  hDmjjnah->SetFillColor(50);
  hDmjjnah->SetFillStyle(1000);
  hDmjjnah->GetXaxis()->SetTitle("#Delta M_{JJ} [GeV]");

  c1.cd();
}
