///////////////////////////////////////////////////////////////////
//
//  J42LCIO
//
//  Jupiter to LCIO Converer
//
//$Id: J42LCIO.cxx,v 1.7 2007/10/05 05:23:30 tyosioka Exp $ 
//  
//////////////////////////////////////////////////////////////////
 
#include "lcio.h"

#include "IO/LCWriter.h"
#include "EVENT/LCIO.h"
#include "DATA/LCFloatVec.h"
#include "DATA/LCIntVec.h"
 
#include "IMPL/LCEventImpl.h"
#include "IMPL/LCRunHeaderImpl.h"
#include "IMPL/LCCollectionVec.h"
#include "IMPL/SimCalorimeterHitImpl.h"
#include "IMPL/SimTrackerHitImpl.h"
#include "IMPL/MCParticleImpl.h"
#include "IMPL/LCFlagImpl.h"
#include "IMPL/LCTOOLS.h"
#include "IMPL/TPCHitImpl.h"
#include "UTIL/LCRelationNavigator.h"

#include "S4BreakPoint.h"
#include "S4GENMCTruthBuf.h"
#include "S4CALAddress.h"
 
#include <cstdlib>
#include <iostream>
#include <sstream>
#include <map>

using namespace std;
using namespace lcio;

typedef map<int, MCParticleImpl*> J42LCIO_MCParticle_Map_t;
J42LCIO_MCParticle_Map_t gJ42LCIO_MCParticles;
vector<int> MCParticles_trackID;

#include "G4Event.hh"
#include "G4Trajectory.hh"

#include "JSFSteer.h"
#include "JSFJ4.h"
#include "JSFJ4RunManager.h"
#include "J42LCIO.h"

#include "S4VTXExactHit.h"
#include "S4ITExactHit.h"
#include "S4TPCExactHit.h"
#include "S4CALHit.h"
#include "S4CALExactHit.h"
#include "S4PrimaryVertex.h"
#include "S4PrimaryParticle.h"

#include "S4TPCPostHit.h"
#include "S4CALPreHit.h"

#include "S4CALPostHit.h"
#include "S4MUDExactHit.h"
#include "S4MUDPreHit.h"

IO::LCWriter *flcWrt=0;        
IMPL::LCRunHeaderImpl* frunHdr=0; 

double gDeltaRayCut = 1.e-3;

bool rejectThisEvent;

ClassImp(J42LCIO)

//_____________________________________________________________________________
  J42LCIO::J42LCIO(const char *name, const char *title)
    : JSF2LCIO(name,title),fJSFJ4(0),fBufp(0),fMCVec(0)
{
  const Char_t *detname=gJSF->Env()->GetValue("J42LCIO.DetectorName","gld_v2");
  fDetectorName=string(detname);

}

//_____________________________________________________________________________
J42LCIO::~J42LCIO()
{
}

//_____________________________________________________________________________
Bool_t J42LCIO::Initialize()
{
  return JSF2LCIO::Initialize();
  //  fOutputFileName=gJSF->Env()->GetValue("JSF2LCIO.OutputFileName","jsf.slcio");

  // create sio writer
  //  flcWrt = LCFactory::getInstance()->createLCWriter() ;
       
  //  flcWrt->open( fOutputFileName.Data() , LCIO::WRITE_NEW )  ;

  //  return kTRUE;
}

//_____________________________________________________________________________
Bool_t J42LCIO::Process(Int_t nev)
{

  rejectThisEvent = false;

// 
  fJSFJ4 = static_cast<JSFJ4*>(gJSF->FindModule("JSFJ4"));
  fBufp= static_cast<JSFJ4Buf *>(fJSFJ4->EventBuf());
  
  // we need to use the implementation classes here
  LCEventImpl *evt = new LCEventImpl() ;
  
  evt->setRunNumber(  gJSF->GetRunNumber()   ) ;
  evt->setEventNumber( gJSF->GetEventNumber() ) ;
  evt->setDetectorName( fDetectorName.Data() ) ;
  
  SetMCParticles(evt);
  
  //   cerr << " will call SetVTXHits " << endl;
  SetVTXHits(evt);
  
  //   cerr << " will call SetITHits " << endl;
  SetBITHits(evt);
  SetFITHits(evt);
  
  //   cerr << " will call SetTPCHits " << endl;
  SetTPCHits(evt);
  
  //   cerr << " will call SetCALHits " << endl;
  std::multimap<Int_t,S4CALExactHit*> calhitmap = MakeCALHitsMap();
  SetCALHits(evt, calhitmap, kTRUE);  // for EM CAL
  SetCALHits(evt, calhitmap, kFALSE); // for HD CAL
  
  //   cerr << " will call SetTPCHits " << endl;
  SetMUDHits(evt);
  
  // write the event to the file
  //   cerr << " will write event " << endl;
  if (!rejectThisEvent) flcWrt->writeEvent( evt ) ;

  // dump the event to the screen
  LCTOOLS::dumpEvent( evt ) ;
  
  //   cerr << " will delete event object " << endl;
  delete evt;
  
  return kTRUE;
  
}

//_____________________________________________________________________________
// Revised version of SetMCParticles

void J42LCIO::SetMCParticles(IMPL::LCEventImpl *evt)
{

  fMCVec = new LCCollectionVec( LCIO::MCPARTICLE );
  gJ42LCIO_MCParticles.clear();
  MCParticles_trackID.clear();
  
  JSFJ4Buf  *bufp = GetBufferPointer();
  
  // --- loop over for the primary particles
  TObjArray *vtxp = bufp->GetComponent("PrimaryVertices");
  TIter next(vtxp);
  S4PrimaryVertex *v=0;
  while( (v=(S4PrimaryVertex*)next() ) ) {
    Int_t ip=0;
    TObjArray pp(*(v->GetParticles()));

    // Then loop over all the MC particles and fill information
    // except parent-daughter relationship and decay point
    for(Int_t i = 0; i < v->GetNumberOfParticles(); i++) {
      
      S4PrimaryParticle *p=(S4PrimaryParticle*)pp.UncheckedAt(i);
      
      MCParticleImpl *mcp=new MCParticleImpl();
      mcp->setPDG(p->GetParticleID());
      float pmom[3] = { p->GetMomentum().Px(),
			p->GetMomentum().Py(),
			p->GetMomentum().Pz()};

      /* not necessary for the primary partilce
      // --- Throw away Delta Ray
      float momp = sqrt(pmom[0]*pmom[0]+pmom[1]*pmom[1]+pmom[2]*pmom[2]);
      std::cout << "delta ray " << momp << " " << p->GetParticleID() << std::endl;
      if ( (TMath::Abs(p->GetParticleID())==11 ||
	    TMath::Abs(p->GetParticleID())==22) && 
	   momp < 1.e-4 ) continue;
      //	   momp < gDeltaRayCut ) continue;
      */

      mcp->setMomentum( pmom ) ;
      mcp->setMass((float)(p->GetMass()) ) ;
      mcp->setCreatedInSimulation(kFALSE);
      mcp->setCharge(p->GetCharge());
      
      // Variables to contain vertex and end points.
      double vtx[3] = {0.,0.,0.};
      double ep[3] = {9999.,9999.,9999.};
      
      // Get vertex and end points, if a corresponding BreakPoint is available.
      // Otherwise zeros are filled.
      Int_t tid = p->GetTrackID();
      S4BreakPoint *brkptr  = S4BreakPoint::GetBreakPointPtr(tid);
      
      if (brkptr!=NULL) {
	// Get a point of generation (vertex)
	vtx[0] = (double)brkptr->GetPosition().X()*10. ;
	vtx[1] = (double)brkptr->GetPosition().Y()*10. ; 
	vtx[2] = (double)brkptr->GetPosition().Z()*10. ;
      } 
      
      mcp->setVertex( vtx );
      // Provisionaly set the end point
      mcp->setEndpoint( ep );
      
      //gJ42LCIO_MCParticles[-i]=mcp;
      gJ42LCIO_MCParticles.insert(pair<int,MCParticleImpl*>(i,mcp));
      MCParticles_trackID.push_back(tid);


    } // end of loop for i (all the MC particles)
    // ------------------------------------------------------------------------


    if ( ip == 0 ){ // --- set tracker associated MC particles
      SetITMCParticles(bufp);
      SetVTXMCParticles(bufp);
      SetTPCMCParticles(bufp);
      SetTPCPostMCParticles(bufp);
    }


    // Then again loop over all the MC particles and find daughter,
    // then associate parent-daughter relationship and get decay point
    // (decay point can be identified only from creation point of the daughters)
    Int_t nparticles = gJ42LCIO_MCParticles.size();
    for (int i=0;i<nparticles;i++) {

      //    MCParticleImpl* mcpt = gJ42LCIO_MCParticles[i];
      MCParticleImpl* mcpt = gJ42LCIO_MCParticles.find(i)->second;
      int tid = MCParticles_trackID[i];

      S4BreakPoint *brkptr = S4BreakPoint::GetBreakPointPtr(tid);
      if (brkptr!=NULL) { // not the primary particles

	int parent_tid = brkptr->GetParentID();
	if ( parent_tid!=0 ) {

	  int parent_key;	  
	  for (parent_key=0;parent_key<MCParticles_trackID.size();parent_key++)
	  if  (MCParticles_trackID[parent_key]==parent_tid) break;
	  //	  std::vector<Int_t>::iterator qq =
	  //	    find(MCParticles_trackID.begin(),MCParticles_trackID.end(),parent_tid);
	  //	  if ( qq == MCParticles_trackID.end() ) continue;
	  //	  parent_key = parent_tid;

	  //	MCParticleImpl* mcpt_parent = gJ42LCIO_MCParticles[parent_key]
	  MCParticleImpl* mcpt_parent 
	    = gJ42LCIO_MCParticles.find(parent_key)->second;


	  // Set decay point of the parent (= creation point of the daughter)
	  if ( !mcpt_parent ){
	    std::cout << "No parent !!! " 
		      << parent_key << " " << parent_tid << std::endl;
	    // --- temporary treatment ----------------------------------------
	    Int_t nnp = gJ42LCIO_MCParticles.size();
	    S4BreakPoint *mcptbp = S4BreakPoint::GetBreakPointPtr(parent_tid);
	    if (!mcptbp){
	      std::cout << "No parent break point !!!" << std::endl;
	      continue; // <- don't forget !!
	    }
	    else{
	      MCParticleImpl *mcp=new MCParticleImpl();
	      mcp->setPDG(mcptbp->GetParticleID());
	      float pmom[3] = { mcptbp->GetFourMomentum().Px(),
				mcptbp->GetFourMomentum().Py(),
				mcptbp->GetFourMomentum().Pz()};

      // --- Throw away Delta Ray
      float momp = sqrt(pmom[0]*pmom[0]+pmom[1]*pmom[1]+pmom[2]*pmom[2]);
      if ( (TMath::Abs(mcptbp->GetParticleID())==11||
	    TMath::Abs(mcptbp->GetParticleID())==22)&& 
	   //	   momp < 1.e-4 ) continue;
	   momp < gDeltaRayCut ) continue;

	      mcp->setMomentum( pmom ) ;
	      mcp->setMass( mcptbp->GetFourMomentum().M() ) ;
	      mcp->setCreatedInSimulation(kFALSE);
	      mcp->setCharge(mcptbp->GetCharge());
	      
	      // Variables to contain vertex and end points.
	      double vtx[3] = {0.,0.,0.};
	      double ep[3] = {9999.,9999.,9999.};
	      
      // Get vertex and end points, if a corresponding BreakPoint is available.
      // Otherwise zeros are filled.
      // Get a point of generation (vertex)
	      vtx[0] = (double)mcptbp->GetPosition().X() *10.;
	      vtx[1] = (double)mcptbp->GetPosition().Y() *10.; 
	      vtx[2] = (double)mcptbp->GetPosition().Z() *10.;
	      
	      mcp->setVertex( vtx );
	      // Provisionaly set the end point
	      mcp->setEndpoint( ep );
	      
	      gJ42LCIO_MCParticles.insert(pair<int,MCParticleImpl*>(nnp,mcp));
	      MCParticles_trackID.push_back(mcptbp->GetTrackID());
	      mcpt_parent = mcp;
	      nnp++;	
	      
	    }
	  }
	  // --- end of temporary treatment -----------------------------------

	  mcpt->addParent( mcpt_parent );

	  if (mcpt_parent->getEndpoint()[0]==9999) {
	    double ep_parent[3];
	    ep_parent[0] = (double)brkptr->GetPosition().X() *10.;
	    ep_parent[1] = (double)brkptr->GetPosition().Y() *10.;
	    ep_parent[2] = (double)brkptr->GetPosition().Z() *10.;
	    mcpt_parent->setEndpoint( ep_parent );
	  }

	}
      } else { // brkptr valid block
	double ep[3] = {0,0,0};
	mcpt->setEndpoint(ep);
      }
      
    } // end of loop for all the particles
    // ------------------------------------------------------------------------
    std::cout << "end of loop for all the particles " << std::endl;

    for (Int_t j=0; j<gJ42LCIO_MCParticles.size();j++){
      std::cout << "List of MC particle : "
		<< gJ42LCIO_MCParticles.find(j)->first << " "
		<< MCParticles_trackID[j] << " " 
		<< gJ42LCIO_MCParticles.find(j)->second->getPDG() << " "
		<< std::endl;
    }

    // ------------------------------------------------------------------------
    float vvtx[3];
    float eep[3];
    float mmom[3];
    float finalparticles_etot = 0;
    for (int i=0;i<gJ42LCIO_MCParticles.size();i++) {
      //    MCParticle* mcpt = gJ42LCIO_MCParticles[i];
      MCParticle* mcpt = gJ42LCIO_MCParticles.find(i)->second;
      fMCVec->push_back(mcpt);
      vvtx[0] = mcpt->getVertex()[0];
      vvtx[1] = mcpt->getVertex()[1];
      vvtx[2] = mcpt->getVertex()[2];
      eep[0] = mcpt->getEndpoint()[0];
      eep[1] = mcpt->getEndpoint()[1];
      eep[2] = mcpt->getEndpoint()[2];
      mmom[0] = mcpt->getMomentum()[0];
      mmom[1] = mcpt->getMomentum()[1];
      mmom[2] = mcpt->getMomentum()[2];
      float vtxr = sqrt(vvtx[0]*vvtx[0]+vvtx[1]*vvtx[1]+vvtx[2]*vvtx[2]);
      float epr = sqrt(eep[0]*eep[0]+eep[1]*eep[1]+eep[2]*eep[2]);
      float mom = sqrt(mmom[0]*mmom[0]+mmom[1]*mmom[1]+mmom[2]*mmom[2]);
      int parentPDG = -999999;
      if (mcpt->getParents().size()!=0) 
	parentPDG = mcpt->getParents()[0]->getPDG();
      
      cout << "FINAL TEST " << i
         << " "           << mcpt->getPDG()
         << " "           << parentPDG
         << " E="         << mcpt->getEnergy()
         << " "           << vtxr
         << " "           << epr
         << " tid="       << MCParticles_trackID[i]
         << " "           << mcpt->getMass()
         << " "           << mom
         << " "           << mcpt->getDaughters().size()
         << endl;

      if (vtxr<1000.&&epr>1000.) {
        finalparticles_etot += mcpt->getEnergy();
      }

      //cout << finalparticles_etot << endl;
    }
    cout << "Energy sum of the final particles : " << finalparticles_etot << endl;
  }
  

  // Append the MCparticle collection to the event
  fNPrimaries=fMCVec->size();
  evt->addCollection( fMCVec, "MCParticle");
  

}



/*
void J42LCIO::SetMCParticles(IMPL::LCEventImpl *evt)
{

  fMCVec = new LCCollectionVec( LCIO::MCPARTICLE );
  gJ42LCIO_MCParticles.clear();

  JSFJ4Buf  *bufp = GetBufferPointer();
  TObjArray *vtxp = bufp->GetComponent("PrimaryVertices");
  TIter next(vtxp);
  S4PrimaryVertex *v=0;

  while( (v=(S4PrimaryVertex*)next() ) ) {
    Int_t ip=0;
    TObjArray pp(*(v->GetParticles()));
    v->Print();
    Int_t np=pp.GetEntries();
    while( ip < np ) {
      S4PrimaryParticle *p=(S4PrimaryParticle*)pp.UncheckedAt(ip);
      //cerr << " ip=" << ip << " np=" << np ;
      //      cerr << " PID=" << p->GetParticleID() << endl;
      MCParticleImpl *mcp=new MCParticleImpl();
      mcp->setPDG(p->GetParticleID());
      float pmom[3] = { p->GetMomentum().Px(),
			p->GetMomentum().Py(),
			p->GetMomentum().Pz()};

      // --- Throw away Delta Ray
      float momp = sqrt(pmom[0]*pmom[0]+pmom[1]*pmom[1]+pmom[2]*pmom[2]);
      if ( (TMath::Abs(p->GetParticleID())==11||
      TMath::Abs(p->GetParticleID())==22)&& 
      momp < gDeltaRayCut ) continue;

      mcp->setMomentum( pmom ) ;
      mcp->setMass((float)(p->GetMass()) ) ;
      mcp->setCreatedInSimulation(kFALSE);
      mcp->setCharge(p->GetCharge());

      // ----------------------------------------------------------
      Int_t tid = p->GetTrackID();
      S4BreakPoint *brkptr  = S4BreakPoint::GetBreakPointPtr(tid);
      double vtx[3] = {0.,0.,0.};
      //brkptr->PrintAll();
      if ( brkptr!=0 ) {
	cout << "brkptr found! " << brkptr << endl;
	vtx[0] = (double)brkptr->GetPosition().X() *10.;
	vtx[1] = (double)brkptr->GetPosition().Y() *10.; 
	vtx[2] = (double)brkptr->GetPosition().Z() *10.;
      } else cout << "brkptr not found." << endl;
      double ep[3] = {0.,0.,0.};
      //cout << v->GetNextVertex() << endl;
      if ( v->GetNextVertex()!=0 ){
	cout << "ep found!" << endl;
	ep[0] = (double)v->GetNextVertex()->GetPosition().X()*10.;
	ep[1] = (double)v->GetNextVertex()->GetPosition().Y()*10.;
	ep[2] = (double)v->GetNextVertex()->GetPosition().Z()*10.;
      } else cout << "ep not found." << endl;
      cout << "vertex " << sqrt(vtx[0]*vtx[0]+vtx[1]*vtx[1]+vtx[2]*vtx[2]) << endl;
      cout << "ep " << sqrt(ep[0]*ep[0]+ep[1]*ep[1]+ep[2]*ep[2]) << endl;
      mcp->setVertex( vtx );
      mcp->setEndpoint( ep );

      fMCVec->push_back(mcp);
      gJ42LCIO_MCParticles[-ip]=mcp;


      //double ep[3] = { 1.111111 , 2.2222222, 3.3333333 } ;
      //d2->setEndpoint( ep ) ;
      //
      //d2->setCreatedInSimulation(kFALSE) ;
      //d2->setBackscatter(kFALSE)         ;
      //d2->setDecayedInTracker(kFALSE)    ;
      //d2->setDecayedInCalorimeter(kFALSE);
      //d2->setHasLeftDetector(kFALSE)     ;
      //d2->setStopped(kFALSE)             ;
      //
      //d2->addParent( d1 );
      //mcVec->push_back( d2 ) ;

      TObjArray *dau=p->GetDaughters();
      if ( dau ) {
	for(Int_t id=0;id<dau->GetEntries();id++) {
	  pp.Add((S4PrimaryParticle*)dau->UncheckedAt(id));
	  np++;
	}
      }
      ip++;
    }
  }

  //  cerr << " End of SetMCParticles " << endl;
  fNPrimaries=fMCVec->size();
  evt->addCollection( fMCVec, "MCParticle");

}
*/

//_____________________________________________________________________________
void J42LCIO::SetCALHits(IMPL::LCEventImpl *evt, std::multimap<Int_t, S4CALExactHit*> calhitmap, Bool_t isEM)
{

  //  std::cout << "SetCALHits ..." << std::endl;

  JSFJ4Buf  *bufp = GetBufferPointer();
  TObjArray *calp  = bufp->GetComponent("CALHits");

  // now add some calorimeter hits
  LCCollectionVec* calVec = new LCCollectionVec( LCIO::SIMCALORIMETERHIT )  ;
  
  // set flag for long format (including position )
  // and PDG and cellid1
  LCFlagImpl chFlag(0) ;
  chFlag.setBit( LCIO::CHBIT_LONG ) ;
  chFlag.setBit( LCIO::CHBIT_PDG ) ;
  chFlag.setBit( LCIO::CHBIT_ID1 ) ;
  calVec->setFlag( chFlag.getFlag()  ) ;

  // --- used cell vector
  std::vector<Int_t> UsedCell;

  TIter next(calp);
  S4CALExactHit *calh=0;
  while( (calh=(S4CALExactHit*)next() ) ) {

    //    for (Int_t i=0; i<UsedCell.size(); i++) 
    //      if ( calh->GetCellID() == UsedCell[i] ) break;
    std::vector<Int_t>::iterator q = 
      find(UsedCell.begin(),UsedCell.end(), calh->GetCellID());
    if ( q != UsedCell.end() ) continue;
    if ( calh->GetEnergyDeposit() <= 0.0 ) continue;
    if ( calh->IsEM() && ! isEM ) continue;
    if ( !calh->IsEM() && isEM ) continue;

    SimCalorimeterHitImpl* hit = new SimCalorimeterHitImpl ;
    UsedCell.push_back( calh->GetCellID() );

    // ------------------------------------------------------------------------
    typedef std::multimap<Int_t,S4CALExactHit*>::iterator SAMECELL;
    std::pair<SAMECELL,SAMECELL> same_cell;
    same_cell = calhitmap.equal_range( calh->GetCellID() );
    SAMECELL itr;
    float edep    = 0.;
    float maxedep = 0.;
    S4CALExactHit *maxcalh=0;
    for (itr = same_cell.first; itr!=same_cell.second; itr++){
      edep += itr->second->GetEnergyDeposit();
      if ( itr->second->GetEnergyDeposit() > maxedep ){
	maxedep = itr->second->GetEnergyDeposit();
	maxcalh = itr->second;
      }
    }
    hit->setEnergy( edep )  ;

    float pos[3] = { 10.0*calh->GetCellPosition().X(),
		     10.0*calh->GetCellPosition().Y(),
		     10.0*calh->GetCellPosition().Z()};
    hit->setPosition( pos ) ;
    //    std::cout << "CAL Hits : " << calh->GetCellID() << " "
    //	      << pos[0] << " " << pos[1] << " " << pos[2] << std::endl;
    hit->setCellID0( calh->GetCellID() ) ;
    Int_t cellid1=0;
    if ( calh->IsEM() ) { cellid1 += 1; }
    hit->setCellID1( S4CALAddress::GetLayerID(calh->GetCellID(),isEM) ) ;

    S4CALPreHit  *phit  = maxcalh->GetPreHitPtr();
    S4TPCPostHit *posth = phit->GetPostHitPtr();
    S4VExactHit  *exh   = (S4VExactHit*)posth;
    if( !exh ) { exh =    (S4VExactHit*)phit ;}
    
    //    cout << "Associate MCParticle to CAL hits" << endl;
    AssociateMCParticle(hit, exh);
    calVec->push_back( hit ) ;
    
  }	  
  

  // --------------------------------------------------------------------------
  if ( isEM ) {
    evt->addCollection( calVec, "GLD_EM");
  }
  else {
    evt->addCollection( calVec, "GLD_HD");
  }
  // --------------------------------------------------------------------------
  
  UsedCell.clear();
  //  std::cout << "End of SetCALHits ..." << std::endl;

}

//_____________________________________________________________________________
void J42LCIO::SetTPCHits(IMPL::LCEventImpl *evt)
{
  JSFJ4Buf  *bufp = GetBufferPointer();

  TObjArray *tpcp  = bufp->GetComponent("TPCHits");
  TObjArray *tpcpp = bufp->GetComponent("TPCPostHits");
  if ( ! tpcp || ! tpcpp ) return ;

  LCCollectionVec* trkVec = new LCCollectionVec( LCIO::SIMTRACKERHIT )  ;
  std::bitset<32>  flag(0);
  flag[ LCIO::CHBIT_LONG ] = 1;
  flag[ LCIO::CHBIT_ID1 ] = 1;
  trkVec->setFlag( (int) flag.to_ulong() );

  TIter next(tpcp);
  S4TPCExactHit *tpch=0;
  while( (tpch=(S4TPCExactHit*)next() ) ) {
    // --- temporary treatment for tracker hit ------------
    S4BreakPoint *brkptr 
      = S4BreakPoint::GetBreakPointPtr(tpch->GetTrackID());
    if ( !brkptr ) continue;
    float pmom[3] = { brkptr->GetFourMomentum().Px(),
		      brkptr->GetFourMomentum().Py(),
		      brkptr->GetFourMomentum().Pz()};
    // --- Throw away Delta Ray
    float momp = sqrt(pmom[0]*pmom[0]+pmom[1]*pmom[1]+pmom[2]*pmom[2]);
    if ( (TMath::Abs(brkptr->GetParticleID())==11||
	  TMath::Abs(brkptr->GetParticleID())==22)&& 
	 momp < gDeltaRayCut ) continue;
    S4BreakPoint *brkptr_p
      = S4BreakPoint::GetBreakPointPtr(brkptr->GetParentID());
    if (brkptr_p!=NULL) {
      float ppmom[3] = { brkptr_p->GetFourMomentum().Px(),
                         brkptr_p->GetFourMomentum().Py(),
                         brkptr_p->GetFourMomentum().Pz()};
      float mompp = sqrt(ppmom[0]*ppmom[0]+ppmom[1]*ppmom[1]+ppmom[2]*ppmom[2]);
    }
    // ----------------------------------------------------
    SimTrackerHitImpl *hit=new SimTrackerHitImpl;
    unsigned cellid = tpch->GetPadRowNumber() ;
    hit->setCellID0(cellid);
    
    double pos[3] = { 10.0*tpch->GetPosition().X(),
		      10.0*tpch->GetPosition().Y(),
		      10.0*tpch->GetPosition().Z()};

    hit->setPosition( pos );
    hit->setdEdx( (float)tpch->GetEnergyDeposit() );
    hit->setTime( (float)tpch->GetTOF() );

    //    cout << "Associate MCParticle to TPC hits" << endl;
    AssociateMCParticle(hit, tpch);
    trkVec->addElement( hit );
  }
  std::cerr << " trkVec size=" << trkVec->size() << std::endl;

  evt->addCollection( trkVec, "GLD_TPC");

}

// ----------------------------------------------------------------------------
void J42LCIO::SetTPCPostMCParticles(JSFJ4Buf *bufp)
{
 
  std::cout << "SetTPCPostMCParticles() is called ... " << std::endl;

  TObjArray *tpcp  = bufp->GetComponent("TPCPostHits");
  Int_t np = gJ42LCIO_MCParticles.size();
  TIter next(tpcp);
  S4TPCPostHit *tpch=0;
  while( (tpch=(S4TPCPostHit*)next() ) ){
    Int_t tid = tpch->GetTrackID();
    std::vector<int>::iterator q = 
      find(MCParticles_trackID.begin(),MCParticles_trackID.end(),tid);
    if ( q != MCParticles_trackID.end() ) continue;

    S4BreakPoint *brkptr = S4BreakPoint::GetBreakPointPtr(tid);
    if (brkptr==NULL) {
      std::cout << "No break point pointer (TPC Post Hit) : " << std::endl;
      continue; // do not store ...
    }
    
    //    std::cout << "np = " << np << std::endl;
    MCParticleImpl *mcp=new MCParticleImpl();
    mcp->setPDG(brkptr->GetParticleID());
    float pmom[3] = { brkptr->GetFourMomentum().Px(),
		      brkptr->GetFourMomentum().Py(),
		      brkptr->GetFourMomentum().Pz()};

    // --- Throw away Delta Ray
    float momp = sqrt(pmom[0]*pmom[0]+pmom[1]*pmom[1]+pmom[2]*pmom[2]);
    if ( (TMath::Abs(brkptr->GetParticleID())==11||
	  TMath::Abs(brkptr->GetParticleID())==22)&& 
	 momp < gDeltaRayCut ) continue;
    S4BreakPoint *brkptr_p
      = S4BreakPoint::GetBreakPointPtr(brkptr->GetParentID());
    if (brkptr_p!=NULL) {
      float ppmom[3] = { brkptr_p->GetFourMomentum().Px(),
                         brkptr_p->GetFourMomentum().Py(),
                         brkptr_p->GetFourMomentum().Pz()};
      float mompp = sqrt(ppmom[0]*ppmom[0]+ppmom[1]*ppmom[1]+ppmom[2]*ppmom[2]);
    }
    mcp->setMomentum( pmom ) ;
    mcp->setMass( brkptr->GetFourMomentum().M() ) ;
    mcp->setCreatedInSimulation(kFALSE);
    mcp->setCharge(brkptr->GetCharge());
	
    // Variables to contain vertex and end points.
    // Get vertex and end points, if a corresponding BreakPoint is available.
    // Otherwise zeros are filled.
    double vtx[3] = {0.,0.,0.};
    double ep[3] = {9999.,9999.,9999.};
    
    // Get a point of generation (vertex)
    vtx[0] = (double)brkptr->GetPosition().X()*10. ;
    vtx[1] = (double)brkptr->GetPosition().Y()*10. ; 
    vtx[2] = (double)brkptr->GetPosition().Z()*10. ;

    mcp->setVertex( vtx );
    // Provisionaly set the end point
    mcp->setEndpoint( ep );
    
    //gJ42LCIO_MCParticles[-i]=mcp;
    gJ42LCIO_MCParticles.insert(pair<int,MCParticleImpl*>(np,mcp));
    MCParticles_trackID.push_back(tid);
    np++;	
    
  } // end of loop for i (all the MC particles)

  std::cout << "SetTPCPostMCParticles is done ... "
	    << gJ42LCIO_MCParticles.size() << std::endl;

}

// ----------------------------------------------------------------------------
void J42LCIO::SetVTXMCParticles(JSFJ4Buf *bufp)
{
 
  std::cout << "SetVTXMCParticles() is called ... " << std::endl;

  TObjArray *tpcp  = bufp->GetComponent("VTXHits");
  Int_t np = gJ42LCIO_MCParticles.size();
  TIter next(tpcp);
  S4VTXExactHit *tpch=0;
  while( (tpch=(S4VTXExactHit*)next() ) ){
    Int_t tid = tpch->GetTrackID();
    std::vector<int>::iterator q = 
      find(MCParticles_trackID.begin(),MCParticles_trackID.end(),tid);
    if ( q != MCParticles_trackID.end() ) continue;

    S4BreakPoint *brkptr = S4BreakPoint::GetBreakPointPtr(tid);
    if (brkptr==NULL) {
      std::cout << "No break point pointer (VTX) : " << std::endl;
      continue; // do not store ...
    }
    
    //    std::cout << "np = " << np << std::endl;
    MCParticleImpl *mcp=new MCParticleImpl();
    mcp->setPDG(brkptr->GetParticleID());
    float pmom[3] = { brkptr->GetFourMomentum().Px(),
		      brkptr->GetFourMomentum().Py(),
		      brkptr->GetFourMomentum().Pz()};

    // --- Throw away Delta Ray
    float momp = sqrt(pmom[0]*pmom[0]+pmom[1]*pmom[1]+pmom[2]*pmom[2]);
    if ( (TMath::Abs(brkptr->GetParticleID())==11||
	  TMath::Abs(brkptr->GetParticleID())==22)&& 
	 momp < gDeltaRayCut ) continue;
    S4BreakPoint *brkptr_p
      = S4BreakPoint::GetBreakPointPtr(brkptr->GetParentID());
    if (brkptr_p!=NULL) {
      float ppmom[3] = { brkptr_p->GetFourMomentum().Px(),
                         brkptr_p->GetFourMomentum().Py(),
                         brkptr_p->GetFourMomentum().Pz()};
      float mompp = sqrt(ppmom[0]*ppmom[0]+ppmom[1]*ppmom[1]+ppmom[2]*ppmom[2]);
    }
    mcp->setMomentum( pmom ) ;
    mcp->setMass( brkptr->GetFourMomentum().M() ) ;
    mcp->setCreatedInSimulation(kFALSE);
    mcp->setCharge(brkptr->GetCharge());
	
    // Variables to contain vertex and end points.
    // Get vertex and end points, if a corresponding BreakPoint is available.
    // Otherwise zeros are filled.
    double vtx[3] = {0.,0.,0.};
    double ep[3] = {9999.,9999.,9999.};
    
    // Get a point of generation (vertex)
    vtx[0] = (double)brkptr->GetPosition().X()*10. ;
    vtx[1] = (double)brkptr->GetPosition().Y()*10. ; 
    vtx[2] = (double)brkptr->GetPosition().Z()*10. ;

    mcp->setVertex( vtx );
    // Provisionaly set the end point
    mcp->setEndpoint( ep );
    
    //gJ42LCIO_MCParticles[-i]=mcp;
    gJ42LCIO_MCParticles.insert(pair<int,MCParticleImpl*>(np,mcp));
    MCParticles_trackID.push_back(tid);
    np++;	
    
  } // end of loop for i (all the MC particles)

  std::cout << "SetVTXMCParticles is done ... "
	    << gJ42LCIO_MCParticles.size() << std::endl;

}

// ----------------------------------------------------------------------------
void J42LCIO::SetITMCParticles(JSFJ4Buf *bufp)
{
 
  std::cout << "SetITMCParticles() is called ... " << std::endl;

  TObjArray *tpcp  = bufp->GetComponent("ITHits");
  Int_t np = gJ42LCIO_MCParticles.size();
  TIter next(tpcp);
  S4ITExactHit *tpch=0;
  while( (tpch=(S4ITExactHit*)next() ) ){
    Int_t tid = tpch->GetTrackID();
    std::vector<int>::iterator q = 
      find(MCParticles_trackID.begin(),MCParticles_trackID.end(),tid);
    if ( q != MCParticles_trackID.end() ) continue;

    S4BreakPoint *brkptr = S4BreakPoint::GetBreakPointPtr(tid);
    if (brkptr==NULL) {
      std::cout << "No break point pointer (IT) : " << std::endl;
      continue; // do not store ...
    }
    
    //    std::cout << "np = " << np << std::endl;
    MCParticleImpl *mcp=new MCParticleImpl();
    mcp->setPDG(brkptr->GetParticleID());
    float pmom[3] = { brkptr->GetFourMomentum().Px(),
		      brkptr->GetFourMomentum().Py(),
		      brkptr->GetFourMomentum().Pz()};

    // --- Throw away Delta Ray
    float momp = sqrt(pmom[0]*pmom[0]+pmom[1]*pmom[1]+pmom[2]*pmom[2]);
    if ( (TMath::Abs(brkptr->GetParticleID())==11||
	  TMath::Abs(brkptr->GetParticleID())==22)&& 
	 momp < gDeltaRayCut ) continue;
    S4BreakPoint *brkptr_p
      = S4BreakPoint::GetBreakPointPtr(brkptr->GetParentID());
    if (brkptr_p!=NULL) {
      float ppmom[3] = { brkptr_p->GetFourMomentum().Px(),
                         brkptr_p->GetFourMomentum().Py(),
                         brkptr_p->GetFourMomentum().Pz()};
      float mompp = sqrt(ppmom[0]*ppmom[0]+ppmom[1]*ppmom[1]+ppmom[2]*ppmom[2]);
    }
    mcp->setMomentum( pmom ) ;
    mcp->setMass( brkptr->GetFourMomentum().M() ) ;
    mcp->setCreatedInSimulation(kFALSE);
    mcp->setCharge(brkptr->GetCharge());
	
    // Variables to contain vertex and end points.
    // Get vertex and end points, if a corresponding BreakPoint is available.
    // Otherwise zeros are filled.
    double vtx[3] = {0.,0.,0.};
    double ep[3] = {9999.,9999.,9999.};
    
    // Get a point of generation (vertex)
    vtx[0] = (double)brkptr->GetPosition().X()*10. ;
    vtx[1] = (double)brkptr->GetPosition().Y()*10. ; 
    vtx[2] = (double)brkptr->GetPosition().Z()*10. ;

    mcp->setVertex( vtx );
    // Provisionaly set the end point
    mcp->setEndpoint( ep );
    
    //gJ42LCIO_MCParticles[-i]=mcp;
    gJ42LCIO_MCParticles.insert(pair<int,MCParticleImpl*>(np,mcp));
    MCParticles_trackID.push_back(tid);
    np++;	
    
  } // end of loop for i (all the MC particles)

  std::cout << "SetITMCParticles is done ... "
	    << gJ42LCIO_MCParticles.size() << std::endl;

}

// ----------------------------------------------------------------------------
void J42LCIO::SetTPCMCParticles(JSFJ4Buf *bufp)
{
 
  std::cout << "SetTPCMCParticles() is called ... " << std::endl;

  TObjArray *tpcp  = bufp->GetComponent("TPCHits");
  Int_t np = gJ42LCIO_MCParticles.size();
  TIter next(tpcp);
  S4TPCExactHit *tpch=0;
  while( (tpch=(S4TPCExactHit*)next() ) ){
    Int_t tid = tpch->GetTrackID();
    std::vector<int>::iterator q = 
      find(MCParticles_trackID.begin(),MCParticles_trackID.end(),tid);
    if ( q != MCParticles_trackID.end() ) continue;

    S4BreakPoint *brkptr = S4BreakPoint::GetBreakPointPtr(tid);
    if (brkptr==NULL) {
      std::cout << "No break point pointer (TPC) : " << std::endl;
      continue; // do not store ...
    }
    
    //    std::cout << "np = " << np << std::endl;
    MCParticleImpl *mcp=new MCParticleImpl();
    mcp->setPDG(brkptr->GetParticleID());
    float pmom[3] = { brkptr->GetFourMomentum().Px(),
		      brkptr->GetFourMomentum().Py(),
		      brkptr->GetFourMomentum().Pz()};

    // --- Throw away Delta Ray
    float momp = sqrt(pmom[0]*pmom[0]+pmom[1]*pmom[1]+pmom[2]*pmom[2]);
    if ( (TMath::Abs(brkptr->GetParticleID())==11||
	  TMath::Abs(brkptr->GetParticleID())==22)&& 
	 momp < gDeltaRayCut ) continue;
    S4BreakPoint *brkptr_p
      = S4BreakPoint::GetBreakPointPtr(brkptr->GetParentID());
    if (brkptr_p!=NULL) {
      float ppmom[3] = { brkptr_p->GetFourMomentum().Px(),
                         brkptr_p->GetFourMomentum().Py(),
                         brkptr_p->GetFourMomentum().Pz()};
      float mompp = sqrt(ppmom[0]*ppmom[0]+ppmom[1]*ppmom[1]+ppmom[2]*ppmom[2]);
    }
    mcp->setMomentum( pmom ) ;
    mcp->setMass( brkptr->GetFourMomentum().M() ) ;
    mcp->setCreatedInSimulation(kFALSE);
    mcp->setCharge(brkptr->GetCharge());
	
    // Variables to contain vertex and end points.
    // Get vertex and end points, if a corresponding BreakPoint is available.
    // Otherwise zeros are filled.
    double vtx[3] = {0.,0.,0.};
    double ep[3] = {9999.,9999.,9999.};
    
    // Get a point of generation (vertex)
    vtx[0] = (double)brkptr->GetPosition().X()*10. ;
    vtx[1] = (double)brkptr->GetPosition().Y()*10. ; 
    vtx[2] = (double)brkptr->GetPosition().Z()*10. ;

    mcp->setVertex( vtx );
    // Provisionaly set the end point
    mcp->setEndpoint( ep );
    
    //gJ42LCIO_MCParticles[-i]=mcp;
    gJ42LCIO_MCParticles.insert(pair<int,MCParticleImpl*>(np,mcp));
    MCParticles_trackID.push_back(tid);
    np++;	
    
  } // end of loop for i (all the MC particles)

  std::cout << "SetTPCMCParticles is done ... "
	    << gJ42LCIO_MCParticles.size() << std::endl;

}


//_____________________________________________________________________________

void J42LCIO::AssociateMCParticle(SimCalorimeterHitImpl *hit, S4VExactHit *exact)
{

  Int_t tid=exact->GetTrackID();
  int key;
  for (key=0;key<MCParticles_trackID.size();key++)
    if (MCParticles_trackID[key]==tid) break;
  if (tid==0 ||key>=MCParticles_trackID.size() ) {
    cout << "In AssociateMCParticle (cal hit) : tid is zero!";
    cout << " PID=" << exact->GetPDGCode() << " " << tid << " " << key << endl;
  } else {
    MCParticleImpl *mcp = gJ42LCIO_MCParticles[key];
    hit->addMCParticleContribution(mcp, (float)exact->GetEnergyDeposit(),
				   (float)exact->GetTOF(),
				   (int)exact->GetPDGCode());
  }

}


//_____________________________________________________________________________
void J42LCIO::AssociateMCParticle(SimTrackerHitImpl *hit, S4VExactHit *exact)
{

 Int_t tid=exact->GetTrackID();
  int key;
  bool check=kFALSE;
  for (key=0;key<MCParticles_trackID.size();key++)
    if (MCParticles_trackID[key]==tid) break;
  if (tid==0 ||key>=MCParticles_trackID.size() ) {
    cout << "In AssociateMCParticle (trk hit) : tid is zero!";
    cout << " PID=" << exact->GetPDGCode() << " " << tid << " " << key << endl;
  } else {
    MCParticleImpl *mcp = gJ42LCIO_MCParticles[key];
    hit->setMCParticle( mcp );
  }

}

//_____________________________________________________________________________
void J42LCIO::SetVTXHits(IMPL::LCEventImpl *evt)
{
  JSFJ4Buf  *bufp = GetBufferPointer();

  TObjArray *vtxp  = bufp->GetComponent("VTXHits");
  if ( ! vtxp ) return ;

  LCCollectionVec* trkVec = new LCCollectionVec( LCIO::SIMTRACKERHIT )  ;
  std::bitset<32>  flag(0);
  flag[ LCIO::CHBIT_LONG ] = 1;
  flag[ LCIO::CHBIT_ID1 ] = 1;
  trkVec->setFlag( (int) flag.to_ulong() );

  TIter next(vtxp);
  S4VTXExactHit *vtxh=0;
  while( (vtxh=(S4VTXExactHit*)next() ) ) {
    // --- temporary treatment for tracker hit ------------
    S4BreakPoint *brkptr 
      = S4BreakPoint::GetBreakPointPtr(vtxh->GetTrackID());
    if ( !brkptr ) continue; 
    float pmom[3] = { brkptr->GetFourMomentum().Px(),
		      brkptr->GetFourMomentum().Py(),
		      brkptr->GetFourMomentum().Pz()};
    // --- Throw away Delta Ray
    float momp = sqrt(pmom[0]*pmom[0]+pmom[1]*pmom[1]+pmom[2]*pmom[2]);
    if ( (TMath::Abs(brkptr->GetParticleID())==11||
	  TMath::Abs(brkptr->GetParticleID())==22)&& 
	 momp < gDeltaRayCut ) continue;
    S4BreakPoint *brkptr_p
      = S4BreakPoint::GetBreakPointPtr(brkptr->GetParentID());
    if (brkptr_p!=NULL) {
      float ppmom[3] = { brkptr_p->GetFourMomentum().Px(),
                         brkptr_p->GetFourMomentum().Py(),
                         brkptr_p->GetFourMomentum().Pz()};
      float mompp = sqrt(ppmom[0]*ppmom[0]+ppmom[1]*ppmom[1]+ppmom[2]*ppmom[2]);
    }
    // ----------------------------------------------------
    SimTrackerHitImpl *hit=new SimTrackerHitImpl;
    unsigned cellid = vtxh->GetLayerID() ;
    hit->setCellID0(cellid);
    
    double pos[3] = { 10.0*vtxh->GetPosition().X(),
		      10.0*vtxh->GetPosition().Y(),
		      10.0*vtxh->GetPosition().Z()};

    hit->setPosition( pos );
    hit->setdEdx( (float)vtxh->GetEnergyDeposit() );
    hit->setTime( (float)vtxh->GetTOF() );
    //hit->setMCParticle( vtxh->GetTrackID() );
    //    cerr << " VTX Hit ID=" << vtxh->GetLayerID() << endl;
    //    cout << "Associate MCParticle to VTX hits" << endl;
    AssociateMCParticle(hit, vtxh);
    trkVec->addElement( hit );
  }

  evt->addCollection( trkVec, "GLD_VTX");

}

//_____________________________________________________________________________
void J42LCIO::SetBITHits(IMPL::LCEventImpl *evt)
{
  JSFJ4Buf  *bufp = GetBufferPointer();

  TObjArray *itp  = bufp->GetComponent("ITHits");
  if ( ! itp ) return ;

  LCCollectionVec* trkVec = new LCCollectionVec( LCIO::SIMTRACKERHIT )  ;
  std::bitset<32>  flag(0);
  flag[ LCIO::CHBIT_LONG ] = 1;
  flag[ LCIO::CHBIT_ID1 ] = 1;
  trkVec->setFlag( (int) flag.to_ulong() );


  TIter next(itp);
  S4ITExactHit *ith=0;
  while( (ith=(S4ITExactHit*)next() ) ) {
    if ( ith->GetSide() ) continue; // 0 : BIT, 1 : FIT
    // --- temporary treatment for tracker hit ------------
    S4BreakPoint *brkptr 
      = S4BreakPoint::GetBreakPointPtr(ith->GetTrackID());
    if ( !brkptr ) continue; 
    float pmom[3] = { brkptr->GetFourMomentum().Px(),
		      brkptr->GetFourMomentum().Py(),
		      brkptr->GetFourMomentum().Pz()};
    // --- Throw away Delta Ray
    float momp = sqrt(pmom[0]*pmom[0]+pmom[1]*pmom[1]+pmom[2]*pmom[2]);
    if ( (TMath::Abs(brkptr->GetParticleID())==11||
      TMath::Abs(brkptr->GetParticleID())==22)&& 
     momp < gDeltaRayCut ) continue;
    S4BreakPoint *brkptr_p
      = S4BreakPoint::GetBreakPointPtr(brkptr->GetParentID());
    if (brkptr_p!=NULL) {
      float ppmom[3] = { brkptr_p->GetFourMomentum().Px(),
                         brkptr_p->GetFourMomentum().Py(),
                         brkptr_p->GetFourMomentum().Pz()};
      float mompp = sqrt(ppmom[0]*ppmom[0]+ppmom[1]*ppmom[1]+ppmom[2]*ppmom[2]);
    }
    // ----------------------------------------------------
    SimTrackerHitImpl *hit=new SimTrackerHitImpl;
    unsigned cellid = ith->GetLayerID() ;
    hit->setCellID0(cellid);
    
    double pos[3] = { 10.0*ith->GetPosition().X(),
		      10.0*ith->GetPosition().Y(),
		      10.0*ith->GetPosition().Z()};

    hit->setPosition( pos );
    hit->setdEdx( (float)ith->GetEnergyDeposit() );
    hit->setTime( (float)ith->GetTOF() );
    //    cout << "Associate MCParticle to IT hits" << endl;
    AssociateMCParticle(hit, ith);
    trkVec->addElement( hit );
  }

  evt->addCollection( trkVec, "GLD_BIT");

}

//_____________________________________________________________________________
void J42LCIO::SetFITHits(IMPL::LCEventImpl *evt)
{
  JSFJ4Buf  *bufp = GetBufferPointer();

  TObjArray *itp  = bufp->GetComponent("ITHits");
  if ( ! itp ) return ;

  LCCollectionVec* trkVec = new LCCollectionVec( LCIO::SIMTRACKERHIT )  ;
  std::bitset<32>  flag(0);
  flag[ LCIO::CHBIT_LONG ] = 1;
  flag[ LCIO::CHBIT_ID1 ] = 1;
  trkVec->setFlag( (int) flag.to_ulong() );


  TIter next(itp);
  S4ITExactHit *ith=0;
  while( (ith=(S4ITExactHit*)next() ) ) {
    if ( !ith->GetSide() ) continue; // 0 : BIT, 1 : FIT
    // --- temporary treatment for tracker hit ------------
    S4BreakPoint *brkptr 
      = S4BreakPoint::GetBreakPointPtr(ith->GetTrackID());
    if ( !brkptr ) continue; 
    float pmom[3] = { brkptr->GetFourMomentum().Px(),
		      brkptr->GetFourMomentum().Py(),
		      brkptr->GetFourMomentum().Pz()};
    // --- Throw away Delta Ray
    float momp = sqrt(pmom[0]*pmom[0]+pmom[1]*pmom[1]+pmom[2]*pmom[2]);
    if ( (TMath::Abs(brkptr->GetParticleID())==11||
      TMath::Abs(brkptr->GetParticleID())==22)&& 
     momp < gDeltaRayCut ) continue;
    S4BreakPoint *brkptr_p
      = S4BreakPoint::GetBreakPointPtr(brkptr->GetParentID());
    if (brkptr_p!=NULL) {
      float ppmom[3] = { brkptr_p->GetFourMomentum().Px(),
                         brkptr_p->GetFourMomentum().Py(),
                         brkptr_p->GetFourMomentum().Pz()};
      float mompp = sqrt(ppmom[0]*ppmom[0]+ppmom[1]*ppmom[1]+ppmom[2]*ppmom[2]);
    }
    // ----------------------------------------------------
    SimTrackerHitImpl *hit=new SimTrackerHitImpl;
    unsigned cellid = ith->GetLayerID() ;
    hit->setCellID0(cellid);
    
    double pos[3] = { 10.0*ith->GetPosition().X(),
		      10.0*ith->GetPosition().Y(),
		      10.0*ith->GetPosition().Z()};

    hit->setPosition( pos );
    hit->setdEdx( (float)ith->GetEnergyDeposit() );
    hit->setTime( (float)ith->GetTOF() );
    //    cout << "Associate MCParticle to IT hits" << endl;
    AssociateMCParticle(hit, ith);
    trkVec->addElement( hit );
  }

  evt->addCollection( trkVec, "GLD_FIT");

}

//_____________________________________________________________________________
void J42LCIO::SetMUDHits(IMPL::LCEventImpl *evt)
{
  JSFJ4Buf  *bufp = GetBufferPointer();

  TObjArray *mudp  = bufp->GetComponent("MUDHits");
  if ( ! mudp ) return ;
  //  cerr << " Number of mud hits is " << mudp->GetEntries() << endl;

  LCCollectionVec* trkVec = new LCCollectionVec( LCIO::SIMTRACKERHIT )  ;
  std::bitset<32>  flag(0);
  flag[ LCIO::CHBIT_LONG ] = 1;
  flag[ LCIO::CHBIT_ID1 ] = 1;
  trkVec->setFlag( (int) flag.to_ulong() );


  TIter next(mudp);
  S4MUDExactHit *mudh=0;
  while( (mudh=(S4MUDExactHit*)next() ) ) {
    SimTrackerHitImpl *hit=new SimTrackerHitImpl;
    unsigned cellid = mudh->GetLayerID() ;
    if ( mudh->GetEnergyDeposit() <= 0.0 ) continue;
    //    cerr << " mudh .. cellid=" << cellid << endl;
    //    cerr << " PID=" << mudh->GetPDGCode() ;
    //    cerr << " Edep=" << mudh->GetEnergyDeposit() ;
    //    cerr << endl;

    hit->setCellID0(cellid);
    
    double pos[3] = { 10.0*mudh->GetPosition().X(),
		      10.0*mudh->GetPosition().Y(),
		      10.0*mudh->GetPosition().Z()};

    hit->setPosition( pos );
    hit->setdEdx( (float)mudh->GetEnergyDeposit() );
    hit->setTime( (float)mudh->GetTOF() );
    //    AssociateMCParticle(hit, mudh);  // Link to MCParticle is not ready yet!
    //    cout << "Associate MCParticle to MU hits" << endl;
    AssociateMCParticle(hit, mudh);
    trkVec->addElement( hit );
  }

  evt->addCollection( trkVec, "GLD_MUD");

}

//_____________________________________________________________________________
Bool_t J42LCIO::BeginRun(Int_t nrun)
{

   frunHdr = new LCRunHeaderImpl ;
   frunHdr->setRunNumber( nrun ) ;
   string detName("GLD_V1")  ;
   frunHdr->setDetectorName( detName ) ;
                                                                               
   stringstream description ;
   description << " run: " << nrun <<" First implementation of lcio in JSF" ;
   frunHdr->setDescription( description.str()  ) ;

   string calName("GLD_CAL") ;
   frunHdr->addActiveSubdetector( calName ) ;
       
   string vtxName("GLD_VTX") ;
   frunHdr->addActiveSubdetector( vtxName ) ;
   string itName("GLD_IT") ;
   frunHdr->addActiveSubdetector( itName ) ;
   string tpcName("GLD_TPC") ;
   frunHdr->addActiveSubdetector( tpcName ) ;

   frunHdr->parameters().setValue( "SimulationProgram" , "jsfj4" ) ;
   IntVec iv(3) ;
   iv[0] = 1 ;
   iv[1] = 2 ;
   iv[2] = 3 ;
   frunHdr->parameters().setValues( "SomeIndices" , iv ) ;
                                                                               
   flcWrt->writeRunHeader( frunHdr ) ;

   return kTRUE;
}

//_____________________________________________________________________________
Bool_t J42LCIO::EndRun()
{
  return JSF2LCIO::EndRun();
}

//_____________________________________________________________________________
Bool_t J42LCIO::Terminate()
{
  return JSF2LCIO::Terminate();
}


//_____________________________________________________________________________
std::multimap<Int_t, S4CALExactHit*> J42LCIO::MakeCALHitsMap()
{

  std::multimap<Int_t, S4CALExactHit*> calhitmap;

  JSFJ4Buf  *bufp = GetBufferPointer();
  TObjArray *calp  = bufp->GetComponent("CALHits");
  TIter next(calp);
  S4CALExactHit *calh=0;
  while( (calh=(S4CALExactHit*)next() ) ) {
    //    std::cout << "Making Calorimeter hit map ..." << std::endl;
    calhitmap.insert(std::pair<Int_t,S4CALExactHit*>(calh->GetCellID(),calh));
  }

  return calhitmap;

}

