//*************************************************************************
//* ----------------
//* S4TPCMCTruth Class
//* ----------------
//* (Description)
//* (Requires)
//*	TObject:
//*	TClonesArray:
//*	TPCUniversalRecordHeader:
//*	TPCRawADCDatum:
//* (Provides)
//*	S4TPCMCTruthBuf		: raw data buffer
//*	S4TPCMCTruth		: steering class
//* (Update Record)
//*	2004/06/02  A.Yamaguchi	Original version.
//*************************************************************************

#include "S4TPCMCTruth.h"
#include "S4TPCExactHit.h"
#include "S4TPCPostHit.h"
#include "S4TPCMCTruthBuf.h"
#include <iostream>
using namespace std;

//#define __DEBUG__

ClassImp(S4TPCMCTruth)

//=====================================================================
//* constructor -------------------------------------------------------

S4TPCMCTruth::S4TPCMCTruth(const Char_t *fname, 
                           const Char_t *name,
                           const Char_t *title)
            : S4VMCTruth(name, title),
              fInputFileName((Char_t *)fname)
{
   fEventBuf = new S4TPCMCTruthBuf(this);
}

//=====================================================================
//* destructor --------------------------------------------------------

S4TPCMCTruth::~S4TPCMCTruth()
{
#ifdef __DEBUG__
   cerr << " S4TPCMCTruth will be deleted...fEventBuf is " 
        << (Int_t)fEventBuf << endl;
#endif
   if(fEventBuf) delete fEventBuf;
   if(!fInputStream) fInputStream->close();
}

//=====================================================================
//* BeginRun  --------------------------------------------------------

Bool_t S4TPCMCTruth::BeginRun(Int_t nrun)
{
   return S4VMCTruth::BeginRun(nrun);
}

//=====================================================================
//* Process   --------------------------------------------------------

Bool_t S4TPCMCTruth::Process(Int_t ev)
{
   S4VMCTruth::Process(ev);
   S4TPCMCTruthBuf *bufp = (S4TPCMCTruthBuf *)fEventBuf;

   if ( fFile->IsWritable() ) {
      if (bufp->GetInputBufPtr()) {
         bufp->GetExactHitsPtr()->Clear();
         TObjArray *ehitbufp   = bufp->GetExactHitsPtr(); 
         TIter next(bufp->GetInputBufPtr());
         TObject *objp = 0;
         while ((objp = next())) ehitbufp->Add(objp);
#ifdef __DEBUG__
         cerr << " TPCHitsPtr = " << (void *)bufp->GetInputBufPtr() << endl;
         cerr << " Nhits      = " << bufp->GetInputBufPtr()->GetEntries() << endl;
#endif
        
      } else {
         bufp->GetExactHitsPtr()->Delete();
         if (bufp->GetPostHitsPtr()) bufp->GetPostHitsPtr()->Delete();
         bufp->ReadFile();
      }
#ifdef __DEBUG__
      cerr << "S4TPCMCTruth::Process nhits =" 
           << bufp->GetExactHitsPtr()->GetEntries() << endl;
#endif
      bufp->MergeHits();
#ifdef __DEBUG__
      DebugPrint();
#endif
      return kTRUE;
   }
   else {
      cerr << "S4TPCMCTruth::Process ExacthitsBuffer is not updated. " << endl;
      return kTRUE;
   }
}


//=====================================================================
//* DebugPrint -------------------------------------------------------

void S4TPCMCTruth::DebugPrint(const Char_t *)
{
   S4TPCMCTruthBuf *bufp = (S4TPCMCTruthBuf *)fEventBuf;
   TIter next(bufp->GetExactHitsPtr()); 
   S4TPCExactHit *ehitp;
   while ((ehitp = static_cast<S4TPCExactHit *>(next()))) {
      ehitp->DebugPrint();
   }

   TIter nextphit(bufp->GetPostHitsPtr()); 
   S4TPCPostHit *phitp;
   while ((phitp = static_cast<S4TPCPostHit *>(nextphit()))) {
      phitp->DebugPrint();
   }
}
