#ifndef __S4VTXExactHit__
#define __S4VTXExactHit__

///////////////////////////////////////////////////////////////////////
//
// --------------------
//  S4VTXExact Hit
// --------------------
// (Description)
//	VTX Exact Hit class.
//     
// (Update Record)
//	2004/10/25  A.Miyamoto  Modified to inherit from S4VExactHit.
//                              Unit of Energy deposit is now GeV, not eV.
//
//$Id: S4VTXExactHit.h,v 1.8 2006/07/03 07:14:37 miyamoto Exp $
//
///////////////////////////////////////////////////////////////////////

#include <TObject.h>
#include <TVector3.h>

#include "S4VExactHit.h"


class J4VTXPixelHit; 
class J4VTXLayerHit;

//=================================================================
// Constructor & Destructor ---------------------------------------

class S4VTXExactHit : public S4VExactHit 
{
 public:
  S4VTXExactHit();
  S4VTXExactHit(J4VTXPixelHit *ahit);
  S4VTXExactHit(J4VTXLayerHit *ahit);
  S4VTXExactHit(      Int_t     evtNumber   ,
                      Int_t     layerNumber ,
                      Int_t     trackNumber ,
                      Int_t     mothertrackN,
                      Int_t     hitnumber   ,
                      Int_t     pcode       ,
                      Int_t     charge      ,
                const TVector3 &prepos      ,
                const TVector3 &postpos     ,
                const TVector3 &momentum    ,
                      Double_t  totE        ,
                      Double_t  edep        ,
                      Double_t  tof         ,
		      Short_t layerID=0     , 
		      Short_t ladderID=0    ,
		      Short_t sensorID=0    , 
		      Short_t pixelThetaID=0,
		      Short_t pixelPhiID=0);
  S4VTXExactHit(const S4VExactHit &hit, 
		const TVector3 &prepos, 
		const TVector3 &postpos, 
		Short_t layerID=0     , 
		Short_t ladderID=0    ,
		Short_t sensorID=0    , 
		Short_t pixelThetaID=0,
		Short_t pixelPhiID=0);

  virtual ~S4VTXExactHit();

  //===============================================================
  // Getter -------------------------------------------------------

  inline Int_t GetParticleID(){ return GetPDGCode(); }
  inline TVector3  GetInPosition()   { return fPrePosition;}
  inline TVector3  GetOutPosition()  { return fPostPosition;}
  inline TVector3  GetPrePosition()  { return fPrePosition;}
  inline TVector3  GetPostPosition() { return fPostPosition;}

  inline Int_t GetLayerID(){ return (Int_t)fLayerID; }
  inline Int_t GetLadderID(){ return (Int_t)fLadderID; }
  inline Int_t GetSensorID(){ return (Int_t)fSensorID; }
  inline Int_t GetPixelThetaID(){ return (Int_t)fPixelThetaID; }
  inline Int_t GetPixelPhiID(){ return (Int_t)fPixelPhiID; }
  inline virtual Double_t GetLayerR() const {return fR;}
  inline virtual Double_t GetPhi()  const {return fPhi;}
  inline virtual Double_t GetZ()    const {return fZ;}

  inline void SetPrePosition(const TVector3 &x)  { fPrePosition  = x; }
  inline void SetPostPosition(const TVector3 &x) { fPostPosition = x; }
  inline void SetLayerR(Double_t d) { fR   = d; }
  inline void SetPhi(Double_t d)    { fPhi = d; }
  inline void SetZ(Double_t d)      { fZ   = d; }
  void Print();

  ClassDef(S4VTXExactHit, 2)  // S4VTXExactHit class

 private:
  TVector3 fPrePosition;
  TVector3 fPostPosition;
  Double_t fPhi;
  Double_t fZ;
  Double_t fR;

  Short_t    fLayerID;       // Layer ID of Hit
  Short_t    fLadderID;      // Lader ID
  Short_t    fSensorID;      // Sensor ID;
  Short_t    fPixelThetaID;  // Theta ID of pixel;
  Short_t    fPixelPhiID;    // PhiID of pixel;

};

#endif










