#ifndef __S4BabyANALYSIS__
#define __S4BabyANALYSIS__

//*************************************************************************
//* -----------
//* S4BabyAnalysis
//* -----------
//* (Description)
//* 	Clasess to analyze Baby data.
//* (Update Record)
//*	2000/05/16  K.Fujii	Original version.
//*************************************************************************

#include "TObject.h"
#include "TH2.h"
#include "TH1E.h"
#include "TCanvas.h"

#include "S4BabyHitMaker.h"

// TObjNum is a simple container for an integer.
class TObjNum : public TObject {
   private:
     Double_t  num; //! data
   public:
     TObjNum(Double_t i = 0) : num(i) { }
     ~TObjNum() { }
     void    SetNum(Double_t i) { num = i; }
     Double_t GetNum() { return num; }
     void    Print(Option_t *) { Printf("TObjNum = %g", num); }
     Bool_t   IsEqual(TObject *obj) { return num == ((TObjNum*)obj)->num; }
     Bool_t   IsSortable() const { return kTRUE; }
     Int_t    Compare(const TObject *obj) const { 
	if (num > ((TObjNum*)obj)->num) 	return 1;
        else if (num < ((TObjNum*)obj)->num) 	return -1;
        else 					return 0; 
     }
   ClassDef(TObjNum, 1) // Double_t object
};

class S4BabyAnalysis : public JSFModule {
public:
   S4BabyAnalysis(const Char_t *name="S4BabyAnalysis",
		  const Char_t *title="Baby Analysis");
   virtual ~S4BabyAnalysis();

   inline void  SetMode(Int_t mode)    { fMode = mode; }
   inline void  SetChi2Cut(Double_t x) { fChi2Cut = x; }
   inline void  SetNsigCut(Double_t x) { fNsigCut = x; }
   inline void  SaturationOff()        { fSaturation = kFALSE; }
   inline void  SaturationOn()         { fSaturation = kTRUE; }

   Bool_t Initialize();
   Bool_t Process(Int_t ev);
   
   void Draw(Option_t *option="");

private:

   virtual void MonitorHits(Int_t ev);
   virtual void MonitorTracks(Int_t ev);	//
   
   struct THdef1 {
      Char_t   title[256];
      Int_t    nxbin;
      Float_t  xmin;
      Float_t  xmax;
   };
   struct THdef2 {
      Char_t   title[256];
      Int_t    nxbin;
      Float_t  xmin;
      Float_t  xmax;
      Int_t    nybin;
      Float_t  ymin;
      Float_t  ymax;
   };
   
   TH2D	   **hDXvsRes;		//!
   TH2D	   **hDXvsSgt2;		//!
   TH1E	   **hDXvsEff;		//!
   TH1D    **hDMultiScat;       //!
   TH2D    **hXdiff;            //!
   TH1E    **hEff2tk;           //!
   TCanvas  *hcDXvsRes;		//! canvas to draw residual plots
   TCanvas  *hcDXvsSgt2;	//! canvas to draw residual plots
   TCanvas  *hcDXvsEff;		//! canvas to draw residual plots
   TCanvas  *hcDMultiScat;      //! canvas to draw tracslation for Muliple scattering
   THdef2   *hdefDXvsRes;	//! histogram parameters for Res
   THdef2   *hdefDXvsSgt2;	//! histogram parameters for Res
   THdef1   *hdefDXvsEff;	//! histogram parameters for Eff
   THdef1   *hdefDMultiScat;    //! histogram parameters for MultiScat
   THdef2   *hdefXdiff;         //!
   THdef1   *hdefEff2tk;        //!
   

   static const Int_t kNh;	//! # hists
   static const Int_t kZoneX;	//! # X divisions
   static const Int_t kZoneY;	//! # Y divisions
   
   Int_t    fMode;		//
   Double_t fChi2Cut;		//
   Double_t fNsigCut;		//
   Bool_t   fSaturation;	// artificial saturation flag

   ClassDef(S4BabyAnalysis, 1) // Baby Analysis class
};

#endif


