//*************************************************************************
//* ---------------------
//* S4BabyHit Class
//* ---------------------
//* (Description)
//*	Class for a single Baby hit.
//* (Requires)
//*	TObject
//* (Provides)
//*	S4BabyHit		: TObject containing single hit data
//* (Update Record)
//*	2002/08/27  R.Kuboshima
//*************************************************************************

#include "S4BabyHit.h"
#include "TRandom.h"
#include "TMath.h"

#define __TMP__

#ifdef __TMP__
#define __DUMMYDXDRIFT__ 0.01
#define __DUMMYDZA__ 	 5.
#define __DUMMYDZS__	 10.*__DUMMYDXDRIFT__
#endif

ClassImp(S4BabyHit)

Int_t    S4BabyHit::fS4BabyHitSortBy = 0; 
Double_t S4BabyHit::fDXpar[] = { 3.83683e-04, 5.83973e+00, 4.04081e-05, 1.24272e-05 };

//=====================================================================
//* constructor -------------------------------------------------------

S4BabyHit::S4BabyHit(S4BabyExactHit * exacthit)
	 : fExactHitPtr(exacthit), fDistanceToPrevious(9999.)
{	
	fExactHitPtr->SetSmearedHitPtr(this);
}

//=====================================================================
//* destructor --------------------------------------------------------

S4BabyHit::~S4BabyHit() 
{
}
  
  
//=====================================================================
//* SetSortBy ------------------------------------------------------=--

void S4BabyHit::SetSortBy(Char_t *opt )
{
	switch (opt[2]) {
		case 'a':	// sort by "TrackNumber"
			fS4BabyHitSortBy = 0;
			break;
		case 'm': 	// sort by "Time"
			fS4BabyHitSortBy = 1;
			break;
		case 'e': 	// sort by "EnergyDeposit"
			fS4BabyHitSortBy = 2;
			break;
		case 'i': 	// sort by "DriftLength"
			fS4BabyHitSortBy = 3;
			break;
		case 'l': 	// sort by "CellNumber"
			fS4BabyHitSortBy = 4;
			break;
		case 'r': 	// sort by "WireLayer"
			fS4BabyHitSortBy = 5;
			break;
		case 'u': 	// sort by "TrueDriftLength"
			fS4BabyHitSortBy = 6;
			break;
		case 'o': 	// sort by "ZCoordinate"
			fS4BabyHitSortBy = 7;
			break;
		default: 	// sort by "DriftLength"
			fS4BabyHitSortBy = 5;
	}		
}

//=====================================================================
//* DebugPrint --------------------------------------------------------

void S4BabyHit::DebugPrint(const Char_t *opt)
{
	if (!fDone) Update();
	std::cerr.setf(std::ios::right);
	std::cerr.setf(std::ios::fixed);
#if 0
	std::cerr    << " (X,Y) = (" << std::setw(5) << setprecision(3) << fWirePtr->X()
		<< "," << std::setw(6) << setprecision(3) << fWirePtr->Y() << ")"
		<< " d = "  << std::setw(6) << setprecision(3) << fXdrift
		<< " dd = " << std::setw(6) << setprecision(4) << fDXdrift
		<< std::endl;
#else 
	Int_t  i;
	if (TestBit(S4BabyHitWire::kECannotSeparate)) {
		i = 1;
	} else {
		i = 0;
	}
	
	std::cerr	<< " TrackNumber = " << fExactHitPtr->GetTrackID()
		<< " PDGcode = " << fExactHitPtr->GetPDGCode()
		<< " CellNumber = " << fExactHitPtr->GetCellNumber()
		<< " WireNumber = " << fExactHitPtr->GetWireNumber()
		<< " DriftLength = " << fExactHitPtr->GetDriftLen()
		<< " TotalEnergy = " << fExactHitPtr->GetTotalEnergy()
		<< " KillHitFlag = " << i
		<< std::endl;
#endif
		
	std::cerr.unsetf(std::ios::right);
	std::cerr.unsetf(std::ios::fixed);
}

//=====================================================================
//* Update ------------------------------------------------------------

void S4BabyHit::Update()
{
	// if (fDone) return;
	fDone = kTRUE;
	CalculatePosition();
}

//=====================================================================
//* CalculatePosition() -----------------------------------------------

void S4BabyHit::CalculatePosition()
{
	if (fWirePtr) {
		S4BabyExactHit *exacthitp = 0;
		if (fLastHitPtr && TestBit(S4BabyHitWire::kEModifiedByDeltaRay)) {
		   exacthitp = (S4BabyExactHit *)fLastHitPtr->GetExactHitPtr();
		} else {
		   exacthitp = fExactHitPtr;
		}

		fXdrift  = TMath::Abs(exacthitp->GetDriftLen());
#if 1      // if 0, smear by flat & if 1, smear by exp.
	        fDXdrift = TMath::Sqrt( fDXpar[0]*TMath::Exp(-fDXpar[1]*fXdrift)
	                    + fDXpar[2]*fXdrift + fDXpar[3] );
#else
		fDXdrift = __DUMMYDXDRIFT__ ;
#endif
		fXdrift += gRandom->Gaus(0.,0.);
//		fXdrift += gRandom->Gaus(0.,fDXdrift);
		
		fZ	 = fExactHitPtr->GetPosition().Z();
		if (fWirePtr->WhatAmI()) {
			fDZ	 = __DUMMYDZS__;
		} else {
			fDZ	 = __DUMMYDZA__;
		}
		
		fZ	+= gRandom->Gaus(0.,fDZ);
				
	} else {
		std::cerr 	<< "::::: Error in S4BabyHit::CalculatePosition"  << std::endl
			<< " This hit belong to no S4BabyHitWire object!" << std::endl
			<< " Append this to some S4BabyHitWire object!"   << std::endl;
	}
}



