//*************************************************************************
//* ---------------------
//* S4BabyTrack Class
//* ---------------------
//* (Description)
//*	Class to store "S4BabyVectorHit"s for a single track.
//* (Requires)
//*	TObjArray
//* (Provides)
//*	S4BabyTrack		: Array containing "S4BabyVectorHit"s
//* (Update Record)
//*	2002/08/28  R.Kuboshima
//*************************************************************************

#include <iostream>
#include "TMath.h"
#include "S4BabyTrack.h"
#include "S4BabyHitWire.h"
#include "S4BabyHit.h"

#define __TMP__

#ifdef __TMP__
#define __BFIELD__ 30.
#endif

ClassImp(S4BabyTrack)

//=====================================================================
//* constructor -------------------------------------------------------

S4BabyTrack::S4BabyTrack() : TObjArray(200), fDone(kFALSE), fTrackNumber(0),
			     fMotherTrackPtr(0)
{
	SetOwner();
}

//=====================================================================
//* destructor --------------------------------------------------------

S4BabyTrack::~S4BabyTrack()
{
}


//=====================================================================
//* IsSortable --------------------------------------------------------

Bool_t S4BabyTrack::IsSortable() const { return kTRUE; }

//=====================================================================
//* Compare -----------------------------------------------------------

Int_t S4BabyTrack::Compare(const TObject *obj) const
{
	Int_t track  = fTrackNumber;
	Int_t tracko = ((S4BabyTrack *)obj)->fTrackNumber;
   	if (track < tracko) return -1;
   	else if (track > tracko) return 1;
   	else return 0;
}

//=====================================================================
//* Append -----------------------------------------------------------

Int_t S4BabyTrack::Append(S4BabyVectorHit *vectorhit)
{	
	Int_t track = vectorhit->GetTrackID();
	
	// Return value = -1:	new vectorhit.
	//		=  0:	successfully appended.
	//		=  1:	new track.
	
	if (GetEntries() == 0) {
		fTrackNumber     = track;
		vectorhit->fTrackPtr = this;
                Add(vectorhit);
                return 0;
	} else {
		if (track == fTrackNumber) {
			vectorhit->fTrackPtr = this;
                        Add(vectorhit);
                        return 0;
                } else {
                        return 1;
                }
        }
}

//=====================================================================
//* DebugPrint --------------------------------------------------------

void S4BabyTrack::DebugPrint(const Char_t *opt)
{
	std::cerr 	<< "track = " << fTrackNumber << std::endl;
	TIter next(this);
	S4BabyVectorHit *vectorhit;
	while ((vectorhit = (S4BabyVectorHit *)next())) {
		vectorhit->DebugPrint(opt);
	}
}




