#ifndef __S4BabyVECTORHIT__
#define __S4BabyVECTORHIT__

//*************************************************************************
//* ---------------------
//* S4BabyVectorHit Class
//* ---------------------
//* (Description)
//*	Class to store "S4BabyHitWire"s for a single cell.
//* (Requires)
//*	TObjArray
//* (Provides)
//*	S4BabyVectorHit		: Array containing "S4BabyHitWire"s
//* (Update Record)
//*	2002/08/28  R.Kuboshima
//*************************************************************************

class S4BabyTrack;
class S4BabyTrackHit;

#include "S4BabyTrackHit.h"
#include "TAttLockable.h"


class S4BabyVectorHit : public TObjArray, public TAttLockable {
friend class S4BabyTrack;

public:
  S4BabyVectorHit();
  S4BabyVectorHit(const S4BabyVectorHit &vh);
  virtual ~S4BabyVectorHit();

  inline virtual S4BabyTrack *GetTrackPtr   () const { return fTrackPtr;    }
  inline virtual Int_t        GetTrackID() const { return fTrackNumber; }
  inline virtual Int_t	      GetCellNumber () const { return fCellNumber;  }	
  inline virtual void	      SetNExpectedCellHitsCut( Int_t nhits )
  				             { fNExpectedCellHitsCut = nhits; }
  inline virtual void	      Revive()            { fKilled = kFALSE; }
  inline virtual Bool_t       IsKilled()   const  { return fKilled;   }
         virtual Int_t	      Append(S4BabyTrackHit *trackhit);
         virtual void 	      KillHits();
  enum {
      kELooping   = 0x00010000,    // Looper cut.			
  };

private:
  Bool_t 	 fKilled;		// Vector hits are killed or not
  Int_t		 fTrackNumber;		// Track Number.
  Int_t		 fCellNumber;		// cell Number.
  S4BabyTrack    *fTrackPtr;		// pointer to track this belongs to
  Double_t	 fNExpectedCellHitsCut; //

// -----------------------------------------------------------------------
// What follows should move eventually to U4BabyVectorHit
// -----------------------------------------------------------------------
public:
  inline virtual void     SetTrackType(Int_t type) { fTrackType = type; }
  inline virtual Int_t    GetTrackType()
  {
	  if (!fDone) Update();
	  return fTrackType;
  }
  inline virtual Double_t GetChi2()
  {
	  if (!fDone) Update();
	  return fChi2;
  }
  inline virtual Double_t Xat(Double_t y)
  {
	  if (!fDone) Update();
	  return (fa*y + fb);
  }
  inline virtual Double_t SGX2at(Double_t y) 
  { 
	  if (!fDone) Update();    
	  return (fEaa*y*y + 2*fEba*y + fEbb);
  }
  inline virtual Double_t GetSlope()
  {
	  if (!fDone) Update();
          return fa;
  }
  inline virtual Double_t GetIntercept()
  {
	  if (!fDone) Update();
	  return fb;
  }
  inline virtual void     Update() 
  { 
	  fDone = kTRUE;    
	  Fit(); 
  }
  void Fit();

  inline virtual Bool_t  IsSortable() const  { return kTRUE;     }
         virtual Int_t	 Compare(const TObject *obj) const;
  
         virtual void 	 DebugPrint(const Char_t *opt = "Brief");

private:
  Bool_t         fDone;                  // (1,0) = (fit done, not yet)
  Int_t          fTrackType;             // track type
  Double_t       fChi2;                  // chi^2
  Double_t       fa;                     // a when x = a*y + b
  Double_t       fb;                     // b
  Double_t       fEaa;                   // <da*da>
  Double_t       fEba;                   // <db*da>
  Double_t       fEbb;                   // <db*db>
  static Int_t   fSortBy;                // (0,1) = by (Chi2,Time)

// -----------------------------------------------------------------------
 
  ClassDef(S4BabyVectorHit, 1) 		// Baby VectorHit
};

#endif






