//*************************************************************************
//* ---------------------
//* S4CDCTrackHit Class
//* ---------------------
//* (Description)
//*	Class for a single CDC hit.
//* (Requires)
//*	TObject
//* (Provides)
//*	U4CDCTrackHit		: Array containing "U4CDCHit"s
//* (Update Record)
//*	1999/12/04  K.Fujii	Original version.
//*************************************************************************

#include <iostream>
#include <iomanip>
#include <math.h>
#include "S4CDCTrackHit.h"
#include "S4CDCExactHit.h"

ClassImp(S4CDCTrackHit)

//=====================================================================
//* constructor -------------------------------------------------------

S4CDCTrackHit::S4CDCTrackHit(S4CDCExactHit* exacthitp) 
	     : U4CDCTrackHit((S4CDCHit *)exacthitp->GetSmearedHitPtr()),
               fKilled(kFALSE),
               fExactHitPtr(exacthitp),
               fVectorHitPtr(0)
{
	if ( exacthitp->GetDriftLen() < 0 ) {
		SetLeftRight(-1);
	} else {
		SetLeftRight(+1);
	}
}

//=====================================================================
//* destructor --------------------------------------------------------

S4CDCTrackHit::~S4CDCTrackHit()
{
}


//=====================================================================
//* DebugPrint --------------------------------------------------------

void S4CDCTrackHit::DebugPrint(const Char_t *opt)
{
	std::cerr.setf(std::ios::right);
	std::cerr.setf(std::ios::fixed);
	std::cerr	<< "wire = (" << std::setw(1) << GetTrackID()
		<< "," << std::setw(1) << GetCellID()
		<< "," << std::setw(1) << GetWireNumber() << ")";
		
	GetHitPtr()->DebugPrint(opt);
	std::cerr.unsetf(std::ios::right);
	std::cerr.unsetf(std::ios::fixed);
}
