//*************************************************************************
//* -----------
//* S4ITAnalysis
//* -----------
//* (Description)
//* 	Clasess to analyze IT data.
//* (Update Record)
//*	2004/09/01  A.Yamaguchi	Original version.
//*************************************************************************

#include <iostream>

#include "JSFHelicalTrack.h"

#include "S4PrimaryVertex.h"
#include "S4GENMCTruthBuf.h"

#include "S4ITHit.h"
#include "S4ITHitMakerBuf.h"
#include "S4ITAnalysis.h"
#include "S4ITTrackHit.h"
#include "S4ITTrack.h"
#include "S4ITTrackMakerBuf.h"

ClassImp(S4ITAnalysis)

//=====================================================================
//* constructor -------------------------------------------------------

S4ITAnalysis::S4ITAnalysis(const Char_t *name, const Char_t *title)
            : JSFModule(name, title)
{
#ifdef __DEBUG__
   std::cerr << " constructor for S4ITAnalysis is called..." << std::endl;
#endif
}

//=====================================================================
//* destructor --------------------------------------------------------

S4ITAnalysis::~S4ITAnalysis()
{
}

//=====================================================================
//* Initialize --------------------------------------------------------

Bool_t S4ITAnalysis::Initialize()
{
   TDirectory *last = gDirectory;
   gFile->cd("/");
   fTrackHistPtr = new TNtupleD("ithtrack", "", "cl:chi2:t0:pt:cos:ndf:cpa:q:x:y:n:nl");
   fHitHistPtr   = new TNtupleD("ithhit", "", "phi:z:edep:mom:path:nx:ny:nz");

   last->cd();
   return kTRUE;
}

//=====================================================================
//* Process -----------------------------------------------------------

Bool_t S4ITAnalysis::Process(Int_t ev)
{
   MonitorTracks(ev);
#ifdef __DEBUG__
   std::cerr << "Analysis::Process evt " << ev << " is finished" << std::endl;
#endif
   return kTRUE;   
}   

//=====================================================================
//* MonitorTracks -----------------------------------------------------

void S4ITAnalysis::MonitorTracks(Int_t ev)
{
   TDirectory *last = gDirectory;
   gFile->cd("/");

   TObjArray *trackaryp = static_cast<S4ITTrackMakerBuf *>
                         (gJSF->FindModule("S4ITTrackMaker")->EventBuf())
                        ->GetOutputBufPtr();
   TIter nexttrack(trackaryp); 
   TIter nextpv(static_cast<S4GENMCTruthBuf *> 
               (gJSF->FindModule("S4GENMCTruth")->EventBuf())
               ->GetInputBufPtr());

   S4ITTrack *trackp = 0;
   while ((trackp = dynamic_cast<S4ITTrack *>(nexttrack()))) {
      //trackp->SetIPConstraint();
      JSFHelicalTrack *heltrackp = trackp->GetHelicalTrackPtr();
      if (trackp->IsLocked()) continue;

      Int_t trackid = trackp->GetTrackID();
      S4PrimaryVertex   *pvp = 0;
      S4PrimaryParticle *ppp = 0;
      while (!ppp && (pvp = dynamic_cast<S4PrimaryVertex *>(nextpv()))) {
         TIter nextpp(pvp->GetParticles());
         while (!ppp && (ppp = dynamic_cast<S4PrimaryParticle *>(nextpp()))) {
            if (trackid == ppp->GetTrackID()) break;
            TIter nextdr(ppp->GetDaughters());
            while ((ppp = dynamic_cast<S4PrimaryParticle *>(nextdr()))) {
               if (trackid == ppp->GetTrackID()) break;
            }
         }
      }
      nextpv.Reset();

      S4ITExactHit *mcp = !ppp ? dynamic_cast<S4ITTrackHit *>
                                (trackp->At(0))->GetHitPtr()->GetExactHitPtr()
                               : 0;
      
      Double_t cl   = heltrackp->GetCL();
      Double_t chi2 = trackp->GetChi2();
      Double_t t0   = trackp->GetT0();
      TVector3 p    = ppp ? ppp->GetMomentum() : mcp->GetMomentum();
      Double_t pt   = p.Pt();
      Double_t cos  = p.CosTheta();
      Double_t ndf  = trackp->GetNDF();
      Double_t cpa  = heltrackp->GetHelixParameter().kappa;
      Double_t q    = ppp ? ppp->GetCharge() : mcp->GetCharge();
      Double_t x    = trackp->GetPosition().X();
      Double_t y    = trackp->GetPosition().Y();
      Double_t n    = trackp->GetNhits();
      //fTrackHistPtr->Fill(cl, chi2, t0, pt, cos, ndf, cpa, q, x, y, n);

      S4ITTrackHit::SetSortBy("Time");
      trackp->UnSort();
      trackp->Sort();
      TIter nexthit(trackp);
      S4ITTrackHit *trackhitp = 0;
      Int_t nl = 0;
      while ((trackhitp = dynamic_cast<S4ITTrackHit *> (nexthit()))) {
         S4ITHit      *hitp  = trackhitp->GetHitPtr();
         S4ITExactHit *ehitp = hitp->GetExactHitPtr();
         Double_t nx, ny, nz;
         if (trackhitp->IsLocked() && hitp->IsLocked()) {
            nl++;
            nx = ehitp->GetPosition().X();
            ny = ehitp->GetPosition().Y();
            nz = ehitp->GetPosition().Z();
         } else {
            nx = ny = nz = 0.;
         }
         if (trackhitp->IsLocked() && !hitp->IsLocked()) break;
         Double_t phi  = ehitp->GetPhi();
         Double_t z    = ehitp->GetPosition().Z();
         Double_t edep = ehitp->GetEnergyDeposit();
         Double_t mom  = ehitp->GetMomentum().Mag();
         Double_t path = (ehitp->GetPostPosition()
                       - ehitp->GetPrePosition()).Mag();
         fHitHistPtr->Fill(phi, z, edep, mom, path, nx, ny, nz);
      }
      fTrackHistPtr->Fill(cl, chi2, t0, pt, cos, ndf, cpa, q, x, y, n, nl);
   }
   last->cd();
}

//=====================================================================
//* Terminate ---------------------------------------------------------

Bool_t S4ITAnalysis::Terminate()
{
   TDirectory *last = gDirectory;
   gFile->cd("/");
   last->cd();
   return kTRUE;
}

//=====================================================================
//* Draw --------------------------------------------------------------

void S4ITAnalysis::Draw(Option_t * /*opt*/)
{
}

