//*************************************************************************
//* ---------------------
//* S4VTXHit Class
//* ---------------------
//* (Description)
//*	Class for a single VTX hit.
//* (Requires)
//*	TObject
//* (Provides)
//*	S4VTXHit		: TObject containing single hit data
//* (Update Record)
//*	2004/12/15  T.Fujikawa	Original version.
//*************************************************************************
#include <iostream>
#include "S4VTXHit.h"
#include "U4VTXHit.h"
#include "TRandom.h"
using namespace std;

ClassImp(S4VTXHit)

Int_t    S4VTXHit::fgSortBy = 0; 
Double_t S4VTXHit::fgSigDZ     = 0.00028;  // [cm]
Double_t S4VTXHit::fgSigDRPhi  = 0.00028;  // [cm]

//=====================================================================
//* constructor -------------------------------------------------------

S4VTXHit::S4VTXHit(S4VTXExactHit * exacthitp)
         : U4VTXHit(exacthitp->GetLayerID(),
           exacthitp->GetLayerR(),
           exacthitp->GetPhi(),
           exacthitp->GetPosition().Z()),
           fExactHitPtr(exacthitp)
{ 
   exacthitp->SetSmearedHitPtr(this);
}

//=====================================================================
//* destructor --------------------------------------------------------

S4VTXHit::~S4VTXHit() 
{
}
  
//=====================================================================
//* SetSortBy ------------------------------------------------------=--

void S4VTXHit::SetSortBy(Char_t *opt)
{
   switch (opt[2]) {
      case 'a':    // sort by "TrackID"
         fgSortBy = 0;
         break;

      case 'm':    // sort by "Time"
      default:
         fgSortBy = 1;
         break;
   }
}

//=====================================================================
//* Compare -----------------------------------------------------------

Int_t S4VTXHit::Compare(const TObject *obj) const
{
   switch (fgSortBy) {
      case 0: { // sort by track ID
         Int_t trackid  = fExactHitPtr->GetTrackID();
         Int_t trackido = dynamic_cast<const S4VTXHit *>(obj)
                                           ->fExactHitPtr->GetTrackID();
         return (trackid < trackido ? -1 : (trackid > trackido ? 1 : 0));
      }
      case 1:
      default: {
         Double_t tof  = fExactHitPtr->GetTOF();
         Double_t tofo = dynamic_cast<const S4VTXHit *>(obj)
                                          ->fExactHitPtr->GetTOF();
         return (tof < tofo ? -1 : (tof > tofo ? 1 : 0));
      }
   }
}

//=====================================================================
//* Update ------------------------------------------------------------

void S4VTXHit::Update()
{
   U4VTXHit::Update();
}

//=====================================================================
//* DebugPrint --------------------------------------------------------

void S4VTXHit::DebugPrint(const Char_t *)
{
   if (!IsDone()) Update();

   cerr << right << fixed
        << "---- S4VTXHit::DebugPrint ---" << endl
        << " Track ID = " << fExactHitPtr->GetTrackID()         << endl
        << " Etot     = " << fExactHitPtr->GetTotalEnergy()     << endl
        << " LayerID  = " << GetLayerID()                       << endl
        << " LayerR   = " << GetLayerR()                        << endl
        << " R * Phi  = " << GetRPhi() << " +/- " << GetDRPhi() << endl
        << " Z        = " << GetZ()    << " +/- " << GetDZ()    << endl;
}

//=====================================================================
//* CalculatePosition() -----------------------------------------------

void S4VTXHit::CalculatePosition()
{
   SetDRPhi(fgSigDRPhi); 
   SetRPhi(fExactHitPtr->GetLayerR() * fExactHitPtr->GetPhi()
                                     + gRandom->Gaus(0., fgSigDRPhi));
   SetDZ(fgSigDZ);
   SetZ(fExactHitPtr->GetPosition().Z() + gRandom->Gaus(0., fgSigDZ)); 
}


//=====================================================================
//* Print () -----------------------------------------------

void S4VTXHit::Print()
{
  cerr << "S4VTXHit..Parameters for VTX Hit point making " << endl;
  cerr << "     Sigma_{DRPhi} = " << fgSigDRPhi << " cm " << endl;
  cerr << "     Sigma_{DZ}    = " << fgSigDZ << " cm " << endl;

}
