{
//*----------------------------------------------------------------------------
//* ===============
//*  Macro plotdx.C
//* ===============
//*
//* (Description)
//*	A sample macro to display plots created by cdcanl.C.
//* (Usage)
//*	$ jsf -b -q unpack.C	# unpack rawdata and produce runxxx.root.
//*	$ jsf -b -q cdcanl.C	# to analyze runxxx.root.
//*	$ jsf jsf.root		# start jsf in interactive mode
//*	......
//*	root [1] .x plotdx.C
//*
//*	This will display residual plots and opens an additional empty
//*	window as a temporary work canvas.
//*
//* (Update Record)
//*	2000/06/16	K.Fujii		Original version.
//*
//*----------------------------------------------------------------------------

gROOT->Reset();

if( strncmp(gSystem->HostName(),"ccjlc",5)  != 0 ) {
    	if( strncmp(gSystem->Getenv("OSTYPE"),"hpux",4) == 0 ) {
      		gSystem->Load("libCDCUtils.sl");
	} else {
      		gSystem->Load("libCDCUtils.so");
    	}
}

Int_t nZoneX = 5;	// corresponds to # wires
Int_t nZoneY = 4;	// corresponds to # plots/wire

TCanvas *cr = new TCanvas("cr","Residual Plots",20,20,200*nZoneX,200*nZoneY);
cr->SetHighLightColor(5);
cr->SetFillColor(19);
cr->Divide(nZoneX,nZoneY);

TCanvas *c2 = new TCanvas("c2","Work",20,50+200*nZoneY,480,380);

const Int_t ncmds = 5;
Char_t *template[ncmds] = {
	"cr->cd(id++);\
	TH2F *h2 = Res%2.2d;\
	h2->SetMarkerColor(46);\
	h2->Draw();\n",

	"Rebin(Res%2.2d,5,4)->FitSlicesY();\n",

	"cr->cd(id++);\
	TH1D *h1 = Res%2.2d_chbn_1;\
	h1->SetMarkerColor(46);\
	h1->SetMinimum(-0.1);\
	h1->SetMaximum(0.1);\
	h1->Draw();\n",

	"cr->cd(id++);\
	TH1D *h1 = Res%2.2d_chbn_2;\
	h1->SetMarkerColor(46);\
	h1->SetMaximum(0.05);\
	h1->Draw();\n",

	"cr->cd(id++);\
	TH2F *h2 = Res%2.2d;\
	TH1D *h1 = h2->ProjectionY();\
	h1->SetFillColor(42);\
	h1->Fit(\"gaus\");\n"
};

gStyle->SetOptFit();	// to display fit parameters in each window

Int_t id = 1;	// directory counter
Int_t j;
for (j = 0; j < ncmds; j++) {
	Int_t i;
	for (i = 0; i < nZoneX; i++) {
		Char_t command[256];
		sprintf(command,template[j],i,i);
		gROOT->ProcessLine(command);
		cr->Update();
	}
}
c2->cd();
}


