{
#include <fstream.h>
#include <TMath.h>
  gROOT->Reset();

  JSFSteer *jsf = new JSFSteer();
  Int_t runno = jsf->Env()->GetValue("JSFSteer.RunNo",1034);

  Char_t fname[64];
  sprintf(fname,"result_root/jsf%4.4dconstped.root",runno);
  TFile file(fname); // Input file
  jsf->SetInput(file);         

//************************************************
//---------- Parameters for histgram -------------- 

   Double_t xhod;
   Double_t hodcount_max;
   Double_t hodAxishi_plus;

   if (runno >= 2842) {
     xhod = 1500.;       	// Threshold of hodoscope count (Xaxis)
				// which separates "between spills" and "during spill".
     hodcount_max = 100000.;	// upper limit of hodoscope count 
     hodAxishi_plus = 2000.;	// upper range of h2px is ihi + hodAxishi_plus. 
   } else {
     xhod = 5000.;       	// Threshold of hodoscope count (Xaxis)
				// which separates "between spills" and "during spill".
     hodcount_max = 100000.;	// upper limit of hodoscope count 
     hodAxishi_plus = 30000.;	// upper range of h2px is ihi + hodAxishi_plus. 
   }	

//************************************************

//=========Macro generated from canvas: c1/c1
//=========  (Tue Nov 20 21:23:52 2001) by ROOT version3.02/03
   TCanvas *c1 = new TCanvas("c1", "c1",102,26,699,498);
   c1->Range(0,0,1,1);
   c1->SetBorderSize(2);
  
// ------------>Setup
   TPad *c1_1 = new TPad("c1_1", "c1_1",0.01,0.51,0.49,0.99);
   c1_1->Draw();
   c1_1->cd();
   c1_1->Range(-7500,-0.25,67500,0.25);
   c1_1->SetBorderSize(2);

   TH2F *h2  = (TH2F *)gROOT->FindObject("hHodXdif");
   TH1D *h2_px   = h2->ProjectionX("h2px");  
   xhod = h2_px->GetMean();

   Int_t xihod   = h2->GetXaxis()->FindBin(xhod);
   
   TH1D *h2lo_py = h2->ProjectionY("h2lopy",-1,xihod);  
   TH1D *h2hi_py = h2->ProjectionY("h2hipy",xihod,9999);
   h2hi_py->Rebin(2);
   h2lo_py->Rebin(2);
#if 0
   Double_t avlo = h2lo_py->GetMean();
   Double_t avhi = h2hi_py->GetMean();
#else
   h2lo_py->Fit("gaus");
   TF1 *flo = h2lo_py->GetFunction("gaus");
   h2hi_py->Fit("gaus");
   TF1 *fhi = h2hi_py->GetFunction("gaus");
   Double_t avlo  = flo->GetParameter(1);
   Double_t avhi  = fhi->GetParameter(1);
#endif
   
   Double_t xlo  = -0.2 + avlo;
   Double_t xhi  = xlo + 0.4;

   // Refit with adjusted fit windows
   h2hi_py->GetXaxis()->SetRange(h2hi_py->GetXaxis()->FindBin(xlo),
   			    	 h2hi_py->GetXaxis()->FindBin(xhi));
   h2lo_py->GetXaxis()->SetRange(h2lo_py->GetXaxis()->FindBin(xlo),
   			    	 h2lo_py->GetXaxis()->FindBin(xhi));
   h2lo_py->Fit("gaus");
   TF1 *flo = h2lo_py->GetFunction("gaus");
   h2hi_py->Fit("gaus");
   TF1 *fhi = h2hi_py->GetFunction("gaus");
   avlo  = flo->GetParameter(1);
   avhi  = fhi->GetParameter(1);

   Double_t davlo = flo->GetParError(1); 
   Double_t davhi = fhi->GetParError(1); 

   xlo  = -0.2 + avlo;
   xhi  = xlo + 0.4;


   h2_px->GetXaxis()->SetRange(h2_px->GetXaxis()->FindBin(0.),
   			       h2_px->GetXaxis()->FindBin(xhod));
   Int_t ilo  = (Int_t)(h2_px->GetMean());
   Double_t dilo = (h2_px->GetRMS())/sqrt((h2_px->GetEntries())/2.);

   h2_px->GetXaxis()->SetRange(h2_px->GetXaxis()->FindBin(xhod),
   			       h2_px->GetXaxis()->FindBin(hodcount_max));
   Int_t ihi  = (Int_t)(h2_px->GetMean());
   Double_t dihi = (h2_px->GetRMS())/sqrt((h2_px->GetEntries())/2.);

   h2->GetXaxis()->SetRange(h2->GetXaxis()->FindBin(0.),
   			    h2->GetXaxis()->FindBin(ihi + hodAxishi_plus));
   h2->GetYaxis()->SetRange(h2->GetYaxis()->FindBin(xlo),
   			    h2->GetYaxis()->FindBin(xhi));
   h2_px->GetXaxis()->SetRange(h2_px->GetXaxis()->FindBin(0.),
   			       h2_px->GetXaxis()->FindBin(ihi + hodAxishi_plus));
   h2hi_py->GetXaxis()->SetRange(h2hi_py->GetXaxis()->FindBin(xlo),
   			    	 h2hi_py->GetXaxis()->FindBin(xhi));
   h2lo_py->GetXaxis()->SetRange(h2lo_py->GetXaxis()->FindBin(xlo),
   			    	 h2lo_py->GetXaxis()->FindBin(xhi));
   
   Double_t xdiff  = avhi - avlo;
   Double_t dxdiff = sqrt(davhi*davhi + davlo*davlo);
   Double_t idiff  = ihi - ilo;
   Double_t didiff = sqrt(dihi*dihi + dilo*dilo);
   Double_t idiffscaled, didiffscaled;
   if (runno >= 2842) {
     idiffscaled  *= 19.;
     didiffscaled *= 19.;
   } 


// ------------>Primitives in pad: c1_1
   h2->Draw("");
   
   TLatex *tex = new TLatex(ihi - 0.7*hodAxishi_plus ,avhi-0.18,"During beam spill");
   tex->SetTextSize(0.05);
   tex->SetLineWidth(2);
   tex->Draw();
   tex = new TLatex(ilo + 0.1*hodAxishi_plus ,avlo+0.13,"Between beam spill");
   tex->SetTextSize(0.05);
   tex->SetLineWidth(2);
   tex->Draw();
   
   TArrow *arrow = new TArrow(ilo + 0.3*hodAxishi_plus, avlo+0.12,ilo + hodAxishi_plus*0.1 ,avlo+0.05,0.03,"|>");
   arrow->SetFillColor(1);
   arrow->SetFillStyle(1001);
   arrow->Draw();
   arrow = new TArrow(ihi - 0.5*hodAxishi_plus,avhi-0.15,ihi-hodAxishi_plus*0.1 ,avhi-0.05,0.03,"|>");
   arrow->SetFillColor(1);
   arrow->SetFillStyle(1001);
   arrow->Draw();

   c1_1->Modified();
   c1->cd();
  
// ------------>Primitives in pad: c1_2
   c1_2 = new TPad("c1_2", "c1_2",0.51,0.51,0.99,0.99);
   c1_2->Draw();
   c1_2->cd();
   c1_2->Range(-0.250758,-9.60943,0.249845,87.3094);
   c1_2->SetBorderSize(2);

   h2lo_py->SetFillColor(1);
   h2lo_py->SetFillStyle(3004);
   h2lo_py->SetLineWidth(3);
   h2hi_py->SetFillColor(2);
   h2hi_py->SetLineColor(2);
   h2hi_py->SetMaximum(90);
   h2hi_py->Draw();
   h2lo_py->Draw("same");

   c1_2->Modified();
   c1->cd();
  
// ------------>Primitives in pad: c1_3
   c1_3 = new TPad("c1_3", "c1_3",0.01,0.01,0.49,0.49);
   c1_3->Draw();
   c1_3->cd();
   c1_3->Range(-7601.32,-48.0471,68740.6,436.547);
   c1_3->SetBorderSize(2);
   
   h2_px->SetMaximum(400);
   h2_px->Draw("");

//   tex = new TLatex(ilo+hodAxishi_plus,220.404,"Between beam spills");
   tex = new TLatex(ilo+0.4*hodAxishi_plus,300.404,"Between beam spills");
   tex->SetTextSize(0.05);
   tex->SetLineWidth(2);
   tex->Draw();
   Char_t string[1024];
//   sprintf(string,"mean = %d nA",ilo);
   sprintf(string,"mean = %d",ilo);
//   tex = new TLatex(0.0013,205.404,string);
   tex = new TLatex(ilo+0.5*hodAxishi_plus,280.404,string);
   tex->SetTextSize(0.05);
   tex->SetLineWidth(2);
   tex->Draw();
//   tex = new TLatex(ihi-1.5*hodAxishi_plus,140.629,"During beam spill");
   tex = new TLatex(ihi-0.8*hodAxishi_plus,157.917,"During beam spill");
   tex->SetTextSize(0.05);
   tex->SetLineWidth(2);
   tex->Draw();
//   sprintf(string,"mean = %d nA",ihi);
   sprintf(string,"mean = %d",ihi);
//   tex = new TLatex(0.0025,125.629,string);
   tex = new TLatex(ihi-0.7*hodAxishi_plus,132.168,string);
   tex->SetTextSize(0.05);
   tex->SetLineWidth(2);
   tex->Draw();

//   arrow = new TArrow(ihi - hodAxishi_plus,120.627,ihi-hodAxishi_plus*0.5,78.3078,0.03,"|>");
   arrow = new TArrow(ihi-0.5*hodAxishi_plus,122.804,ihi-hodAxishi_plus*0.1,88.3078,0.03,"|>");
   arrow->SetFillColor(1);
   arrow->SetFillStyle(1001);
   arrow->Draw();
//   arrow = new TArrow(0.00177839,197.362,0.00106658,162.018,0.03,"|>");
   arrow = new TArrow(ilo+0.5*hodAxishi_plus,271.980,ilo+hodAxishi_plus*0.1,162.018,0.03,"|>");
   arrow->SetFillColor(1);
   arrow->SetFillStyle(1001);
   arrow->Draw();

   TMarker *marker = new TMarker(xhod,15,23);
   marker->SetMarkerColor(2);
   marker->SetMarkerStyle(23);
   marker->SetMarkerSize(1.7);
   marker->Draw();

   c1_3->Modified();
   c1->cd();
  
// ------------>Primitives in pad: c1
#if 0
   tex = new TLatex(0.574431,0.43,"Hodoscope count vs X_{Diff}");
   tex->SetTextSize(0.05);
   tex->SetLineWidth(2);
   tex->Draw();
#endif
   sprintf(string,"Run %d",runno);
   tex = new TLatex(0.574431,0.41623,string);
   tex->SetTextSize(0.05);
   tex->SetLineWidth(2);
   tex->Draw();
   
   Int_t av  = (Int_t)(avlo*1.e4);
   Int_t dav = (Int_t)(davlo*1.e4);
   sprintf(string,"mean = %d #pm%d #mum (lo)",av,dav);
   tex = new TLatex(0.574431,0.35,string);
   tex->SetTextSize(0.05);
   tex->SetLineWidth(2);
   tex->Draw();
   av  = (Int_t)(avhi*1.e4);
   dav = (Int_t)(davhi*1.e4);
   sprintf(string," = %d #pm%d #mum (hi)",av,dav);
   tex = new TLatex(0.658494,0.305,string);
   tex->SetTextColor(2);
   tex->SetTextSize(0.05);
   tex->SetLineWidth(2);
   tex->Draw();

   av  = (Int_t)(xdiff*1.e4);
   dav = (Int_t)(dxdiff*1.e4);
   sprintf(string,"X_{diff} = %d #pm%d #mum",av,dav);
   tex = new TLatex(0.574431,0.25,string);
   tex->SetTextSize(0.05);
   tex->SetLineWidth(2);
   tex->Draw();
   sprintf(string,"I_{diff} = %d #pm%d",idiff,didiff);
   tex = new TLatex(0.574431,0.19,string);
   tex->SetTextSize(0.05);
   tex->SetLineWidth(2);
   tex->Draw();
   Double_t nentlo = h2lo_py->Integral(0,99999);
   Double_t nenthi = h2hi_py->Integral(0,99999);
#if 0
   cerr << "nentlo = " << nentlo << endl;
   cerr << "nenthi = " << nenthi << endl;
#endif
   sprintf(string,"N_{entries} = %d (lo)",nentlo);
   tex = new TLatex(0.574431,0.12,string);
   tex->SetTextSize(0.05);
   tex->SetLineWidth(2);
   tex->Draw();
   sprintf(string,"= %d (hi)",nenthi);
   tex = new TLatex(0.677594,0.075,string);
   tex->SetTextColor(2);
   tex->SetTextSize(0.05);
   tex->SetLineWidth(2);
   tex->Draw();

   c1->Modified();
   c1->cd();

// ------------>Output
   Char_t psname[64];
   sprintf(psname,"run%4.4d.hodvsXdif.ps",runno);
   c1->Print(psname);

   ofstream out;
   out.open("xdiff_hod.dat",ios::app);
   out << runno <<" "<< idiffscaled << " " << xdiff << " " 
       << didiffscaled << " " << dxdiff << " " 
       << ilo << " " << avlo << " " << ihi << " " << avhi << endl;
   out.close();
}
