{
//**********************************************************
//*   Event display program
//*   
//*   This is an example of display program.
//*   You MUST switch user analysis program in the end 
//*   of this program.
//*
//*   2001/11/19  K.Hoshina
//*
//**********************************************************
	
//*------------------
//* Library Roading 
//*------------------

  gSystem->Load("libS4Utils.so");
  gSystem->Load("libS4Kalman.so");  
  gSystem->Load("libS4Helix.so");  
  gSystem->Load("libS4KalTrack.so");
  gSystem->Load("libU4CDCRawData.so");
  gSystem->Load("libU4CDCClusterMaker.so");
  gSystem->Load("libU4CDCKalDetector.so");
  gSystem->Load("libU4VHitMaker.so");
  gSystem->Load("libU4CDCHitMaker.so");
  gSystem->Load("libU4VTrackMaker.so");
  gSystem->Load("libU4CDCTrackMaker.so");
  gSystem->Load("libU4BabyTrackMaker.so");
  gSystem->Load("libBabyDisplay.so");
  
//*--------------------
//* Define output file 
//*--------------------

  JSFSteer *jsf = new JSFSteer();

  TFile *foutp = new TFile("jsf.root","RECREATE"); //  Output file.
  TFile &fout  = *foutp;

  Int_t run = jsf->Env()->GetValue("JSFSteer.RunNo",778);  
  Char_t fname[64];
  // sprintf(fname,"../../../analysis/prod/run%4.4d.root",run);
  //sprintf(fname,"/proj/cdc/data/baby/2000/beam/unpacked/run%4.4d.root",run);
  // sprintf(fname,"/proj/cdc/data/baby/2000/dedx/unpacked/run%4.4d.root",run);
  // sprintf(fname,"/proj/cdc/data/baby/2001/beam/unpacked/run%4.4d.root",run);
  // sprintf(fname,"run%4.4d.root",run);
  sprintf(fname,"../../../analysis/prod/run%4.4d.root",run);
  TFile file(fname); // Input file
  jsf->SetInput(file);
  jsf->SetOutput(fout);
 
//*------------------------------------------------
//* Set modules, calibration files, cut values etc.
//*------------------------------------------------

  U4CDCFADCCalibration fadccalib("../../../../../data/fadccalib.dat");
  U4CDCClusterMaker *clust = new U4CDCClusterMaker(&fadccalib);
#if 1
  clust->UseConstPed();
#else
  clust->UseVarPed();
#endif
#if 1
  U4CDCCluster::SetSoftThresh(16.);
  U4CDCCluster::SetSplitThresh(20.);
#else
  U4CDCCluster::SetSoftThresh(10.);
  U4CDCCluster::SetSplitThresh(20.);
#endif
#if 0
  U4CDCCluster::SetMinNslots(20); 	// for cluster splitting
#else
  U4CDCCluster::SetMinNslots(100); 	// for cluster splitting
#endif
  U4CDCCluster::SetMaxNslots(250);	// for ph sum
#if 1
  U4CDCCluster::SetIntegrateNslots(20);   //for cluster integration
#endif

  U4CDCTimeToXRelation xtcalib("../../../../../data/xtcalib.dat");
#if 1
  U4CDCXTDatum::SetSigmaMS(0.000);
#else
  U4CDCXTDatum::SetSigmaMS(0.050);
#endif

  U4CDCHitMaker   *hit   = new U4CDCHitMaker(&xtcalib);

  U4BabyTrackMaker *track = new U4BabyTrackMaker();

//*--------------------
//* JSF event rootine
//*--------------------

  jsf->Initialize();

  U4BabyTrackMakerBuf *tkb = (U4BabyTrackMakerBuf *)track->EventBuf();
#if 1
  tkb->SetNsigCut(5.);		// window for residals
  tkb->SetChi2Cut(35.);		// chi2 cut for tracks
  tkb->SetKCNsigCut(3.);	// window for hit point poisoing
  tkb->SetDXpivotCut(0.05);	// DX to be qualified as a pivot
  tkb->SetSlopeCut(0.15);	// cut on track slope
#else
  tkb->SetNsigCut(5.);		// window for residals
  tkb->SetChi2Cut(15.);		// chi2 cut for tracks
  tkb->SetKCNsigCut(3.);	// window for hit point poisoing
  tkb->SetDXpivotCut(0.05);	// DX to be qualified as a pivot
  tkb->SetSlopeCut(999.);	// cut on track slope
#endif

  U4CDCHitMakerBuf *htb = (U4CDCHitMakerBuf *)hit->EventBuf();
#if 1
  htb->SetWidthCut(12.);
  htb->SetPeakCut(30.);
  htb->SetSumCut(200.);
#else
  htb->SetWidthCut(12.);
  htb->SetPeakCut(20.);
  htb->SetSumCut(200.);
#endif

  U4CDCRawData *raw=(U4CDCRawData*)jsf->FindModule("U4CDCRawData");

  jsf->BeginRun(run);   // Set run number

  BabyDisplay  *display = new BabyDisplay();
  
//*******************************************
//*----------!!! ATTENTION !!!-------------
//* Create new useranalysis object as your
//* concrete class object.
//* DON'T CHANGE THE OBJECT-NAME.
//* "useranl" is used by Display program.
//*----------------------------------------

  VUserAnalysis *useranl = new UserAnalysis(display,&fadccalib);
  
  display->Initialize();
  
}

