//**************************************************************************
//* ----------------------------
//* Cluster Buffer
//* ----------------------------
//**************************************************************************

#include "U4CDCCluster.h"
#include "U4CDCClusterBuf.h"
#include "U4CDCClusterMaker.h"
#include "U4CDCFADCCalibration.h"
#include <fstream>
#include <iomanip>

using namespace std;

//--------------------------------------------------------------------------

ClassImp(U4CDCClusterBuf)

U4CDCClusterBuf::U4CDCClusterBuf(U4CDCFADCCalibration *calib,
                             const char *name,const char *title,
                             U4CDCClusterMaker *module) :
        JSFEventBuf(name, title, (JSFModule*)module),
        fDone(kFALSE), fUseVarPed(kTRUE), fNADCData(0), fADCData(0),
        fGlobalClusters(1), fClusters(1),
        fCalib(calib) {}

U4CDCClusterBuf::U4CDCClusterBuf(U4CDCClusterMaker *module,
                             U4CDCFADCCalibration *calib,
                             const char *name, const char *title) :
        JSFEventBuf(name, title, (JSFModule*)module),
        fDone(kFALSE), fUseVarPed(kTRUE), fNADCData(0), fADCData(0),
        fGlobalClusters(1), fClusters(1),
        fCalib(calib) {}

U4CDCClusterBuf::~U4CDCClusterBuf()
{
        fGlobalClusters.Delete();
        fClusters.Delete();
}

void U4CDCClusterBuf::SetADCData(Len_t nadcdata, TClonesArray *adcdata)
{
        fDone     = kFALSE;
        fNADCData = nadcdata;
        fADCData  = adcdata;
        if (fGlobalClusters.GetEntries() > 0)   fGlobalClusters.Delete();
        if (fClusters.GetEntries() > 0)         fClusters.Delete();
}

void U4CDCClusterBuf::SetCalibrationData(U4CDCFADCCalibration *calib)
{
        fDone  = kFALSE;
        fCalib = calib;
}

Bool_t U4CDCClusterBuf::IsInitialized()  const
{
        if (fNADCData > 0) return kTRUE;
        else return kFALSE;
}

Int_t U4CDCClusterBuf::GetNclusters()
{
        if (!fDone) FindClusters();
        return fClusters.GetEntries();
}

TObjArray * U4CDCClusterBuf::GetClusters()
{
        if (!fDone) FindClusters();
        return &fClusters;
}

void U4CDCClusterBuf::UseVarPed()
{
        if (!IsInitialized()) {
                fUseVarPed = kTRUE;
        } else {
                if (!fUseVarPed) {
                        fDone      = kFALSE;
                        fClusters.Delete();
                        fUseVarPed = kTRUE;
                        TIter next(&fGlobalClusters);
                        U4CDCCluster *clp;
                        while ((clp = (U4CDCCluster *)next())) {
                                clp->UseVarPed();
                        }
                }
        }
}

void U4CDCClusterBuf::UseConstPed()
{
        if (!IsInitialized()) {
                fUseVarPed = kFALSE;
        } else {
                if (fUseVarPed) {
                        fDone      = kFALSE;
                        fClusters.Delete();
                        fUseVarPed = kFALSE;
                        TIter next(&fClusters);
                        U4CDCCluster *clp;
                        while ((clp = (U4CDCCluster *)next())) {
                                clp->UseConstPed();
                        }
                }
        }
}

void U4CDCClusterBuf::FindClusters()
{
#if 1
std::cerr << "fNADCData = " << fNADCData << std::endl;
#endif
        if (!IsInitialized()) {
                std::cerr << "ERROR  U4CDCClusterBuf::FindCluster" << std::endl
                     << "...... ADC data not yet initialized." << std::endl;
                return;
        } else if (fDone || fNADCData == 0) {
                return;
        }

        fDone = kTRUE;
        if (fGlobalClusters.GetEntries() == 0) {
                U4CDCCluster *cur = new U4CDCCluster(fCalib,fUseVarPed);
                fGlobalClusters.Add(cur);
                U4CDCRawADCDatum *adcp;
                for (Int_t i = 0; i < fNADCData; i++) {
                        adcp = (U4CDCRawADCDatum *)fADCData->UncheckedAt(i);
                        if (adcp->GetChannelNo() < 0 ||
                        adcp->GetChannelNo() >= U4CDCRawADCDatum::GetNchMax()) {
                                // adcp->DebugPrint();
                                continue; // skip hodoscopes
                        }
                        if (! cur->Append(adcp)) {
                                cur = new U4CDCCluster(fCalib,fUseVarPed);
                                fGlobalClusters.Add(cur);
                                cur->Append(adcp);
                        }
                }
        }

        if (fGlobalClusters.GetEntries() == 0) return;

        TIter next(&fGlobalClusters);
        U4CDCCluster *clp;

        while ((clp = (U4CDCCluster *)next())) {
               clp->Integrate();                // smooth signals
               clp->SplitInto(fClusters);       // split multi-peaks
        }
}

void U4CDCClusterBuf::DebugPrint(const Char_t *opt)
{
        std::cerr    << fClusters.GetEntries()
                << " clusters in this event." << std::endl;
        TIter next(&fClusters);
        U4CDCCluster *clp;
        Int_t n = 0;
        while ((clp = (U4CDCCluster *)next())) {
                std::cerr << std::setw(3) << ++n << " ";
                clp->DebugPrint(opt);
        }
}
