#ifndef __U4ITPADHIT__
#define __U4ITPADHIT__
//*************************************************************************
//* ---------------------
//* U4ITHit Class
//* ---------------------
//* (Description)
//*	Class for a single IT hit.
//* (Requires)
//*	TObject
//* (Provides)
//*	U4ITHit		: TObject containing single hit data
//* (Update Record)
//*	2004/08/07  A.Yamaguchi	Original version.
//*************************************************************************

#include "TObjArray.h"
#include "TAttLockable.h"

class U4ITHit : public TObject, public TAttLockable {

public:
  U4ITHit(Int_t    layerid = -9999,
          Int_t    side    = -9999,
          Double_t x1      = -9999.,
          Double_t x2      = -9999.);
  virtual ~U4ITHit();

  inline virtual Int_t     GetLayerID() const { return fLayerID; }
  inline virtual Int_t     GetSide   () const { return fSide;    }

  inline virtual Bool_t    IsDone    () const { return fDone;    }

  inline virtual Double_t  GetX1     ();
  inline virtual Double_t  GetDX1    ();
  inline virtual Double_t  GetX2     (); 
  inline virtual Double_t  GetDX2    ();
  
  inline virtual void SetX1  (Double_t d) { fX1   = d; }
  inline virtual void SetDX1 (Double_t d) { fDX1  = d; }
  inline virtual void SetX2  (Double_t d) { fX2   = d; }
  inline virtual void SetDX2 (Double_t d) { fDX2  = d; }

  inline virtual void SetDone(Bool_t   b) { fDone = b; }

  static  void        SetSortBy (Char_t *opt);
  virtual Bool_t      IsSortable()                   const;
  virtual Int_t       Compare   (const TObject *obj) const;
  virtual void        DebugPrint(const Char_t *opt = "Brief");

protected:
  virtual void  Update();

private:
  virtual void  CalculatePosition();

private:
  Bool_t   fDone;      // (1,0) = (cluster done, not yet)
  Double_t fX1;        // rphi or x [cm]
  Double_t fDX1;       // error on X1 [cm]
  Double_t fX2;        // z or y [cm]
  Double_t fDX2;       // error on X2 [cm]
  Int_t    fLayerID;   // layer id
  Int_t    fSide;      // side

  static Int_t  fgITHitSortBy;

  ClassDef(U4ITHit, 1)          // IT Anode Hit
};


//=====================================================================
//* GetX1 -------------------------------------------------------------

Double_t U4ITHit::GetX1()
{
   if (!fDone) Update();
    return fX1;
}

//=====================================================================
//* GetDX1 ------------------------------------------------------------

Double_t U4ITHit::GetDX1()
{
   if (!fDone) Update();
   return fDX1;
}

//=====================================================================
//* GetX2 -------------------------------------------------------------

Double_t U4ITHit::GetX2()
{
   if (!fDone) Update();
   return fX2;
}

//=====================================================================
//* GetDX2 ------------------------------------------------------------

Double_t U4ITHit::GetDX2()
{
   if (!fDone) Update();
   return fDX2;
}

#endif

